package did.gui;

import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Graphics;
import java.awt.Polygon;
import java.awt.Color;
import java.awt.Point;
import java.awt.Font;
import java.awt.Stroke;
import java.awt.BasicStroke;
import java.awt.geom.AffineTransform;
import java.awt.geom.PathIterator;

import java.util.Random;
import java.util.Properties;
import java.util.Enumeration;
import java.util.StringTokenizer;

import did.gui.DIDRoiIFC;
import did.xml.DIDNode;
import did.xml.DIDDefaultTreeModel;

/*****************************************************************************************
*<pre>
*File:         DIDBaseROI.java
*Version:      1.1
*Description:  Abstract class from which all specialized ROI classes are derived.  This 
*class implements the DIDRoiIFC interface and inherits from the Java Polygon class.  Any 
*class derived from this one will thus inherit this implementation allowing universal 
*access to all common methods and attributes of the ROI architecture.  This class also 
*contains a reference to a DIDNode.  This DIDNode points to a specific portion of the 
*TreeModel which serves as a starting point from which to set and get all persitable data.
*
*Modification Log:
*Date          Name    Comment
*xx-xx-xxxx    XXX	   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
*</pre>
*@see DIDPolyRoi
*@see DIDRectRoi
******************************************************************************************/

public abstract class DIDBaseRoi extends Polygon implements DIDRoiIFC {
	/********************************************************************************************************/
	/** Attribute used to determine the size of the text used to display coordinate information. */
	protected static int myFontSize = 10;
	/** Attribute used to determine the type of font used to display coordinate information. */
	protected static Font myFont = new Font("Serif", Font.PLAIN, myFontSize);
	/** Attribute used to determine the color used to display coordinate information. */
	protected static Color myFontColor = Color.magenta;
	/** Attribute used to determine the color of each rectangle that marks a vertex. */
	protected Color myVertexColor = new Color(160, 0, 0);
	/** Attribute used to determine the color of the active vertex. */
	protected Color myActiveVertexColor = Color.green;
	/** Attribute used to determine the color of the entire ROI when it does have focus. */
	protected Color myFocusColor = Color.red;
	/** Attribute used to determine the color of the entire ROI when it does not have focus. */
	protected Color myNoFocusColor = Color.blue;
	/** Attribute used to determine the color of the ROI's inside. */
	protected Color myFillColor = null;
	private Color previousFillColor = null;
	/** Attribute used to determine the color of the ROI when it is selected by search function. */
	protected Color mySelectedColor = Color.orange;
	/** Attribute used to determine the color of the arrow showing the logical relationship. */
	protected Color myArrowColor = Color.black;
	/** Integer used to dermine the size and location of the rectangle which marks a vertex. */
	protected static int	CORNER_WIDTH	= 4,
							CORNER_HEIGHT	= 4,
							X_OFFSET	= -2,
							Y_OFFSET	= -2;
	/********************************************************************************************************/
	
	/********************************************************************************************************/
	/** Attribute that holds the current rotation factor. */
	protected double myTheta = 0;
	/** Attribute that points to the active vertex.  Since the Polygon class stores coordinates in two integer
	arrays (xpoints and ypoints), the value of this attribute is used to point to a specific element in each 
	array thus making it the active vertex. */
	protected int myActiveVertex = 0;
	/** Attribute used to determine the  */
	protected int myStatus = WAITING;
	/********************************************************************************************************/
	
	/********************************************************************************************************/
	/** Attribute that holds the current transform for this ROI. The AffineTransform class represents a 2D 
	affine transform that performs a linear mapping from 2D coordinates to other 2D coordinates that preserves 
	the "straightness" and "parallelness" of lines.  */
	protected AffineTransform at = new AffineTransform();
	/** Attribure that hold the current iterator implementation for this ROI.  The PathIterator interface 
	provides the mechanism for objects that implement the Shape interface to return the geometry of their 
	boundary by allowing a caller to retrieve the path of that boundary a segment at a time.  */
	protected PathIterator pi = this.getPathIterator(at);
	/********************************************************************************************************/

	/********************************************************************************************************/
	/** Attribute used to determine if coordinate information should be rendered next to each vertex. */
	protected boolean showCoordinates = true;
	/** Attribute used to determine if the ROI has focus and therefore should be rendered distinctly. */
	protected boolean focus = false;

	/** Attribute used to determine if the ROI has been searched and therefore should be rendered distinctly. */
	protected boolean selected = true;

	/** if true, draw an arrow to the logically next ROI */
	protected boolean myDrawArrow = false;

	/** the next logical node  */
	protected DIDBaseRoi myNextRoi = null;

	/** the previous logical node  */
	protected DIDBaseRoi myPrevRoi = null;

	/** Attribute that references  */
	public DIDNode myAnchorNode = null;
	/********************************************************************************************************/

	/*****************************************************************************************
	* Constructor used to initalize the object.  Initializes the ROI from the specified 
	* parameters.
	* @param tmpX An array of integers corresponding to all the x corrdinates for this ROI.
	* @param tmpY An array of integers corresponding to all the y corrdinates for this ROI.
	* @param tmpN The number of verticies this ROI should contain.
	*****************************************************************************************/   
	public DIDBaseRoi(int tmpX[], int tmpY[], int tmpN) {
		super(tmpX, tmpY, tmpN);
		init();
	}

	/*****************************************************************************************
	* Method used to instantitiate this class.  This constructor takes one argument, x, which
	* preinitializes the number of verticies this ROI should contain.
	* @param x The number of verticies this instance of this class should contain.
	*****************************************************************************************/   
	public DIDBaseRoi(int x) {
		super();
		for (int i = 0; i < x ; i++) { addPoint(0, 0); }
		init();
	}

	/*****************************************************************************************
	* Method used to instatiate the class.
	*****************************************************************************************/   
	public DIDBaseRoi() {
		super();
		init();
	}

	/*****************************************************************************************
	* Method used to initialize the ROI.  Initialization involves setting the focus to true, 
	* creating a new bounding rectangle (important to ensure proper bounds detection) and 
	* loading the vertex rectangle dimensions from the properties file.
	*****************************************************************************************/   
	public void init() { 
		setFocus(true);
		setSelected(false);
		bounds = new Rectangle();
		DIDProperties newProps = DIDProperties.getInstance();
		try {
			CORNER_WIDTH = Integer.parseInt(newProps.getProperty("CORNER_WIDTH"));
			CORNER_HEIGHT = Integer.parseInt(newProps.getProperty("CORNER_HEIGHT"));
			X_OFFSET = Integer.parseInt(newProps.getProperty("X_OFFSET"));
			Y_OFFSET = Integer.parseInt(newProps.getProperty("Y_OFFSET"));
			myFocusColor = stringToColor(newProps.getProperty("FocusColor"));
			myArrowColor = stringToColor(newProps.getProperty("ArrowColor"));
		} catch (NumberFormatException nfe) {
			System.out.println("Error while trying to set Vertex Dimensions from trueviz.properties file.");
			System.out.println(nfe);
		}
	}
	
	/*****************************************************************************************
	* Method used to convert a string representation of an RGB colorset (###,###,### where ###
	* is an integer between 0 and 255) to a java Color object.
	* @param newColor String representation of an RGB color (###,###,###).
	* @return A new Color object if the String can be sucessfully parsed else null.
	*****************************************************************************************/
	private Color stringToColor(String newColor) {
		if (newColor == null) {
			return (null);
		} else if (Character.isLetter(newColor.charAt(0))) {
			if (newColor.equals("black")) {
				return Color.black;
			} else if (newColor.equals("blue")) {
				return Color.blue;
			} else if (newColor.equals("cyan")) {
				return Color.cyan;
			} else if (newColor.equals("darkGray")) {
				return Color.darkGray;
			} else if (newColor.equals("gray")) {
				return Color.gray;
			} else if (newColor.equals("green")) {
				return Color.green;
			} else if (newColor.equals("lightGray")) {
				return Color.lightGray;
			} else if (newColor.equals("magenta")) {
				return Color.magenta;
			} else if (newColor.equals("orange")) {
				return Color.orange;
			} else if (newColor.equals("pink")) {
				return Color.pink;
			} else if (newColor.equals("red")) {
				return Color.red;
			} else if (newColor.equals("white")) {
				return Color.white;
			} else if (newColor.equals("yellow")) {
				return Color.yellow;
			}
		} else if (Character.isDigit(newColor.charAt(0))) {
			StringTokenizer st = new StringTokenizer(newColor, ",", false);
			int ctr = 0;
			int[] colorArray = new int[3];
			while (st.hasMoreTokens() && ctr < 3) {
				colorArray[ctr] = Integer.parseInt(st.nextToken());
				if (colorArray[ctr] > 255) { colorArray[ctr] = 255; }
				if (colorArray[ctr] < 0) { colorArray[ctr] = 0; }
				ctr++;
			}
			return (new Color(colorArray[0], colorArray[1], colorArray[2]));
		}
		return (null);
	}
	
	/*****************************************************************************************
	* Method used to return the current PathIterator implementation for this class.  The 
	* iterator is crucial to the rendering of the ROI as it allows for the application of a 
	* transformation to a shape and returns the geometry of the shapes boundary by allowing a
	* caller to retrieve the path of that boundary a segment at a time.  This transforamtion 
	* does not affect the internal coordinate arrays.
	* @return The current PathIterator implementation for this class.
	*****************************************************************************************/   
	public PathIterator getPathIterator(){
		return this.getPathIterator(at);
	}
	
	/*****************************************************************************************
	* This method renders the object onto whatever graphics object is passed to it as a 
	* parameter.
	* @param g2d A reference to a specific Graphics2D object onto which this ROI should render
	* itself.
	*****************************************************************************************/   
	public abstract void draw(Graphics2D g2d) ;

	public abstract void draw (Graphics2D g2d, float scale);

	public Rectangle getBounds() {
		return ((new Polygon(xpoints, ypoints, npoints)).getBounds());
	}

	public Point getCenterPoint (float scale)  {
		double x, y;
		Rectangle bounds = getBounds();
		x = (bounds.getX()+bounds.getWidth()/2.0)*scale;
		y = (bounds.getY()+bounds.getHeight()/2.0)*scale;

		return (new Point((int)x, (int)y));
	}

	public void drawTriangle (Graphics2D g2d, Point p1, Point p2, int len)  {
                int num = 4, sign;
                int[] x = new int[num];
                int[] y = new int[num];
                double xm, ym, xc, yc;

                        x[0] = x[3] = (int)p2.getX();
                        y[0] = y[3] = (int)p2.getY();

                        if (p2.getY()-p1.getY() == 0)  {
                                sign = (p2.getX()-p1.getX() > 0)? 1:-1;
                                xc = p2.getX() - sign * 10;
                                yc = p2.getY();
                                x[1] = x[2] = (int)xc;
                                y[1] = (int)(yc + 5);
                                y[2] = (int)(yc - 5);
                        } else if (p2.getX()-p1.getX() == 0) {
                                sign = (p2.getY()-p1.getY() > 0)? 1:-1;
                                xc = p2.getX();
                                yc = p2.getY() - sign * 10;
                                x[1] = (int)(xc + 5);
                                x[2] = (int)(xc - 5);
                                y[1] = y[2] = (int)yc;
                        } else {


                        double slope = (p2.getY()-p1.getY())/(p2.getX()-p1.getX(
));
                        sign = (p2.getX()-p1.getX() > 0)? 1:-1;
                        xm = sign*Math.sqrt((10.0*10.0)/(1+slope*slope));
                        ym = (slope * xm);
                        xc = p2.getX()-xm;
                        yc = p2.getY()-ym;

                        slope = -1/slope;
                        sign = (slope > 0)? 1:-1;
                        xm = sign*Math.sqrt((5.0*5.0)/(1+slope*slope));
                        ym = (slope * xm);
                        x[1] = (int)(xc - xm);
                        y[1] = (int)(yc - ym);

                        xm = sign*Math.sqrt((5.0*5.0)/(1+slope*slope));
                        ym = (slope * xm);
                        x[2] = (int)(xc + xm);
                        y[2] = (int)(yc + ym);
                        }

                        g2d.fillPolygon(x, y, num);
	}


	public void drawArrow (Graphics2D g2d, float scale)  {
		Point p1 = getCenterPoint(scale);
		Stroke currStroke = g2d.getStroke();

		g2d.setPaintMode();
		if(getNextRoi() != null)  {
			g2d.setColor(myArrowColor);
			Point p2 = getNextRoi().getCenterPoint(scale);
			drawTriangle(g2d, p1, p2, CORNER_WIDTH*4);
			g2d.fillOval((int)p1.getX() + X_OFFSET,
				(int)p1.getY() + Y_OFFSET,
				CORNER_WIDTH, CORNER_HEIGHT);
			g2d.setStroke(new BasicStroke((float)((BasicStroke)currStroke).getLineWidth()*2.0f));
			g2d.drawLine((int)p1.getX(), (int)p1.getY(), (int)p2.getX(), (int)p2.getY());
			g2d.setStroke(currStroke);
		} else {
			g2d.setColor(myArrowColor);
			g2d.fillOval((int)p1.getX() + X_OFFSET*2,
				(int)p1.getY() + Y_OFFSET*2,
				CORNER_WIDTH*2, CORNER_HEIGHT*2);
		}
	}
	
	/*****************************************************************************************
	* Method used to rotate the ROI.  The attribute myTheta is incremented by the value of 
	* parameter myAmount.  Rotation is accomplished by setting the rotation factor of the 
	* affinetransform used to render the ROI.
	* NOTE: Rotation does NOT change the internal data representation of the ROI.  Only a call
	* to finalizeROI will reset the Polygon's internal data.  As continuous calls to 
	* finalizeROI() will destort the ROI due to the internal coordinate structure only being
	* stored as integers instead of doubles it is recommended that the coordinated be finalized
	* only on a user click. 
	* @param myAmount The amount by which the ROI should rotated.  Negative values rotate to 
	* the left and positive to the right.
	*****************************************************************************************/   
	public void rotate(double myAmount) {
		myTheta = myTheta + myAmount;
		setStatus(DIDRoiIFC.ROTATING);
		at.setToRotation(Math.toRadians(myTheta), xpoints[myActiveVertex], ypoints[myActiveVertex]);
	}

	/*****************************************************************************************
	* Method used to dertermine if a set of coordinates is within the bounds of the first 
	* vertex of the ROI.  This method is used in the creation process to signify to completion
	* of ROI creation.
	* @param x X coordinate to be tested.
	* @param y Y coordinate to be tested.
	* @return True if parameters x & y fall inside the bounds of the first vertex, false if it
	* is not.
	* @see DIDCreationHandler#mouseClicked
	*****************************************************************************************/   
	public boolean isOriginVertex(int x, int y) {
		Rectangle tmpRect = new Rectangle(xpoints[0] + X_OFFSET, ypoints[0] + Y_OFFSET,
			CORNER_WIDTH, CORNER_HEIGHT);
		return tmpRect.contains(x, y);
	}
	
	/*****************************************************************************************
	* Method used to move the active vertex to the specified coordinate.
	* @param x New x position for the active vertex.
	* @param y New y position for the active vertex.
	*****************************************************************************************/   
	public void setActiveVertexCoord(int x, int y) {
		modifyPoint(myActiveVertex, x, y);
	}
	
	/*****************************************************************************************
	* Method used to set the active vertex.  This method should determine if the coordinates
	* passed into it lay within one of the ROIs verticies.  If so, make that vertex the active
	* one and return true, else return false.
	* @param x X coordinate to be iterrigated to see if it lies within the active vertex bounds. 
	* @param y Y coordinate to be iterrigated to see if it lies within the active vertex bounds.
	* @return True if the x,y parameters are within a vertex handle, false if they are not.
	*****************************************************************************************/   
	public boolean setActiveVertex(int x, int y) {
		Rectangle tmpRect = null;
		for (int i = 0; i < npoints; i++) {
			tmpRect = new Rectangle(xpoints[i] + X_OFFSET, ypoints[i] + Y_OFFSET,
				CORNER_WIDTH, CORNER_HEIGHT);
			if (tmpRect.contains(new Point(x, y))) { 
				myActiveVertex = i;
				return(true);
			}
		}
		return(false);
	}

	/*****************************************************************************************
	* Convenience method used move the entire ROI to a new location.  Should be constrained to
	* the active vertex.
	* @param x Integer representing the x coordinate to which the ROI should be moved.
	* @param y Integer representing the y coordinate to which the ROI should be moved.
	*****************************************************************************************/   
	public void setTranslation(int x, int y) {
		translate((x - xpoints[myActiveVertex]), (y - ypoints[myActiveVertex]));
	}


	/*****************************************************************************************
	* Method used to finalize and ROI.  Finalizing means that any pending transforms relating
	* to this ROI (stored in the 'at' attribute) are calculated and a PathIterator is used to 
	* enumertate the shape and set the ROI's internal coordinate structure to the appropriate
	* values (based on the transform).
	* @see #at
	* @see #pi
	*****************************************************************************************/   
	public void finalizeROI() {
		pi = this.getPathIterator(at);
		int n = 0;
		while (pi.isDone() != true) {
			double myArray[] = new double[6];
			int mySegType = pi.currentSegment(myArray);
			switch (mySegType) {
					case pi.SEG_MOVETO : case pi.SEG_LINETO :
						modifyPoint(n, (int)myArray[0], (int)myArray[1]);
					break;
					default :
					break;
			}
			n++;
			pi.next();
		}
		myTheta = 0;
		at.setToRotation(Math.toRadians(myTheta), xpoints[myActiveVertex], ypoints[myActiveVertex]);
	}	

	/*****************************************************************************************
	* Method used to convert ROI statistics into a string that can be returned when the ROI is
	* dumbed to an output stream.
	* @return A string that details the ROI's properties.
	*****************************************************************************************/
	public String toString() {
		String tmpString = new String("Name=" + this.getClass().getName() +
			", Status=" + myStatus + ", #Verticies=" +	npoints + ", Vertex Readout(");
		for (int i = 0; i < npoints ; i++) {
			tmpString = tmpString + (new String(" " + xpoints[i] + "," + ypoints[i]));
		}
		tmpString = tmpString + (new String("), Type=" + this.getType()));
		return(tmpString);
	}

	/*****************************************************************************************
	* Method used to get the type (Rectangle, Polygon) of a particular implementation of this 
	* class.  Since this class is abstract the DIDRoiIFC.BASE value should never be returned.  
	* @return An integer repesenting the type of ROI (Rectangle, Polygon).
	*****************************************************************************************/   
	public int getType() { return DIDRoiIFC.BASE; }

	/*****************************************************************************************
	* Method used to determine whether coordinate information is to be displayed next to each 
	* vertex in the ROI.
	* @param myValue True if coordinate info is to be displayed next to each vertex, false
	* if no information is to be displayed.
	*****************************************************************************************/   
	public void setCoordDisp(boolean myValue) {	showCoordinates = myValue; }

	/*****************************************************************************************
	* Method used to set the focus of a single instance of a ROI.  Only one ROI at a time can
	* have the focus and it is denoted by a different color.
	* @param state A boolean primitive used to determine if the current instance of the ROI 
	* should get the focus.
	*****************************************************************************************/   
	public void setFocus(boolean state) {
		if (focus == state)  return;

		focus = state;
		if (focus) {
			myArrowColor = myFocusColor.darker();
			previousFillColor = myFillColor;
			setFillColor(myFocusColor);
		} else {
			myArrowColor = myNoFocusColor.darker();
			setFillColor(previousFillColor);
		}
	}

	/***********************************************************************
	* Method used to indicate a ROI is selected by search function.
	* selected ROI is denoted by a different color.
	* @param state A boolean primitive used to determine if the current 
	* instance of the ROI is selected.
	***********************************************************************/
	public void setSelected(boolean state) {
		if (selected == state)  return;

		selected = state;
		if (selected) {
			myArrowColor = mySelectedColor.darker();
			previousFillColor = myFillColor;
			setFillColor(mySelectedColor);
		} else {
			myArrowColor = myNoFocusColor.darker();
			setFillColor(previousFillColor);
		}
	}


	/*****************************************************************************************
	* Method used to determine if an ROI has the focus.  Returns the value of attribute focus.
	* @return True if this ROI has the focus, false if it does not.
	*****************************************************************************************/   
	public boolean hasFocus() { return focus; }

	/*****************************************************************************************
	* Method used to set the status of the ROI.  Status is mainly used in the creation of ROIs.
	* @param s An integer that specifies the state of the ROI. WAITING=-1,CONSTRUCTING=0,
	* MOVING=1,RESIZING=2,ROTATING=3,NORMAL=4;
	* @see DIDCreationHandler
	*****************************************************************************************/   
	public void setStatus(int s) { myStatus = s; }

	/*****************************************************************************************
	* Method used to get the Status of the ROI.
	* @return An integer that specifies the state of the ROI. 
	* @see DIDRoiIFC#WAITING
	*****************************************************************************************/   
	public int getStatus() { return myStatus; }

	/*****************************************************************************************
	* Method used to set the color of the ROI when it has the focus.
	* @param c A color object to which the ROI who has the focus should be set.
	*****************************************************************************************/   
	public void setFocusColor(Color c) { myFocusColor = c; }

	/*****************************************************************************************
	* Method used to get the color of the ROI when it has the focus.
	* @return A color object representing the color of the ROI with the focus.
	*****************************************************************************************/   
	public Color getFocusColor() { return myFocusColor; }

	/*****************************************************************************************
	* Method used to set the color of the ROI when it doesn't have the focus.
	* @param c A color object to which the ROI who doesn't have the focus should be set.
	*****************************************************************************************/   
	public void setNoFocusColor(Color c) {
		myNoFocusColor = c;
		myArrowColor = myNoFocusColor.darker();
	}

	/*****************************************************************************************
	* Method used to get the color of the ROI when it doesn't have the focus.
	* @return A color object representing the color of the ROI with out the focus.
	*****************************************************************************************/   
	public Color getNoFocusColor() { return myNoFocusColor; }

	/*****************************************************************************************
	* Method used to set the color of the ROI's inside.
	* @param A color object to which the color of the ROI's inside should be set.
	*****************************************************************************************/   
	public void setFillColor(Color c) {
			myFillColor = c;
	}

	/*****************************************************************************************
	* Method used to get the color of the ROI's inside.
	* @return A color object representing the color of the ROI's inside.
	*****************************************************************************************/   
	public Color getFillColor() { return myFillColor; }

	/*****************************************************************************************
	* Method used to set a reference to the corresponding DIDNode which the ROI is 
	* representing.  This implementation also sets the focus color of the ROI based on the 
	* type of data the anchorNode contains.
	* @param newAnchorNode A reference to the corresponding DIDNode.
	* @see #stringToColor
	*****************************************************************************************/   
	public void setAnchorNode(DIDNode newAnchorNode) { 
		myAnchorNode = newAnchorNode;
		String nodeDataType = myAnchorNode.getDataType();
		DIDProperties newProps = DIDProperties.getInstance();
		Color newColor = stringToColor(newProps.getProperty(nodeDataType + "Color"));
		if (newColor != null) { setNoFocusColor(newColor); }
	}

	/*****************************************************************************************
	* Method used to set a reference to the corresponding DIDNode which the ROI is 
	* representing.  This implementation also sets the focus color of the ROI.
	* @param newAnchorNode A reference to the corresponding DIDNode.
	* @see #stringToColor
	*****************************************************************************************/   
	public void setAnchorNode(DIDNode newAnchorNode, String color) { 
		myAnchorNode = newAnchorNode;
		String nodeDataType = myAnchorNode.getDataType();
		DIDProperties newProps = DIDProperties.getInstance();
		Color newColor = stringToColor(color);
		if (newColor == null) {
			newColor = stringToColor(newProps.getProperty(nodeDataType + "Color"));
		}
		if (newColor != null) { setNoFocusColor(newColor); }
	}

	/*****************************************************************************************
	* Method used to return a reference to the corresponding DIDNode.
	* @return DIDNode referenced in attribute list and which this ROI is visually representing.
	*****************************************************************************************/   
	public DIDNode getAnchorNode() { return myAnchorNode; }

	/***********************************************************************
	* Method used to set a value of myDrawArrow.
	***********************************************************************/
	public void setDrawArrow(boolean da) { myDrawArrow = da; }

	/***********************************************************************
	* Method used to return a value of myDrawArrow.
	* @return boolean myDrawArrow.
	***********************************************************************/
	public boolean getDrawArrow() { return myDrawArrow; }

	/***********************************************************************
	* Method used to set a value of the next ROI.
	***********************************************************************/
	public void setNextRoi(DIDBaseRoi nextRoi) { myNextRoi = nextRoi; }

	/***********************************************************************
	* Method used to return a reference to the next ROI.
	* @return DIDBaseRoi which is the logically next ROI.
	***********************************************************************/
	public DIDBaseRoi getNextRoi() { return myNextRoi; }

	/***********************************************************************
	* Method used to set a value of the previous ROI.
	***********************************************************************/
	public void setPrevRoi(DIDBaseRoi prevRoi) { myPrevRoi = prevRoi; }

	/***********************************************************************
	/***********************************************************************
	* Method used to return a reference to the previous ROI.
	* @return DIDBaseRoi which is the logically previous ROI.
	***********************************************************************/
	public DIDBaseRoi getPrevRoi() { return myPrevRoi; }


	/*****************************************************************************************
	* Method used to synchronize coordinate data between the ROI class and the TreeModel (XML).
	* This method traverses the TreeModel starting from the point of the anchorNode.  From 
	* here is looks for all nodes whose name begins "Vertex" and whose parent node's name begins
	* with the type of data this ROI represents and ends with "Corner".  Once a node is found, 
	* the nodes under this node are exposed and if their name begins with "x" their value is 
	* set to the current x coordinate and vice versa for those that start with "y".  A counter
	* keeps track of where the method is in the internal coordinate structure.
	*****************************************************************************************/   
	public void syncData() {
		if (myAnchorNode != null) { 
			String nodeType = myAnchorNode.getDataType(); 
			int vertexCtr = 0;
			for (Enumeration element = myAnchorNode.breadthFirstEnumeration(); element.hasMoreElements(); ) {
				DIDNode tmpElement = (DIDNode)element.nextElement();
				DIDNode parent = (DIDNode)tmpElement.getParent();
				if (tmpElement.getNodeName().equals("Vertex") && parent.getNodeName().equals(nodeType + "Corners")) {
					for (Enumeration attribute = tmpElement.children(); attribute.hasMoreElements(); ) {
						DIDNode tmpAttribute = (DIDNode)attribute.nextElement();
						if (tmpAttribute.getNodeName().equals("x")) {
							Integer tmpX = new Integer(xpoints[vertexCtr]);
							tmpAttribute.setNodeValue(tmpX.toString());
						} else if (tmpAttribute.getNodeName().equals("y")) {
							Integer tmpY = new Integer(ypoints[vertexCtr]);
							tmpAttribute.setNodeValue(tmpY.toString());
						}
					}
					vertexCtr++;
				}
			}
		}
	}

	/*****************************************************************************************
	* Method used to return the type of data this ROI is reprsenting (Zone, Line, Character).
	* @return An integer (should be a DIDRoiIFC constant) denoting the type of data this
	* ROI represents.
	*****************************************************************************************/   
	public int getNodeType() {
		if (myAnchorNode == null) {
			return DIDRoiIFC.UNSPECIFIED;
		} else {
			String tmpString = myAnchorNode.getDataType();
			int type = DIDDefaultTreeModel.EntityType.nameToType(tmpString);
			return type;
		}
	}


	/*****************************************************************************************
	* Method used to safely modify an element in the base polygons coordinate arrays( xpoints,
	* ypoints).  NOTE: This class should ALWAYS be used to modify the internal arrays.  Never
	* modify them directly as doing so will not correctly update the bounding rectangle thus
	* causing the contains() method to act in unpredictable ways.
	* @param n References the element in the x & y coordinate arrays which are to be modified.
	* @param x New x coordinate.
	* @param y New y coordinate.
	*****************************************************************************************/   
	protected void modifyPoint(int n, int x, int y) {
		// if the bounds already contains x and y
	    if (bounds.contains(x, y)) {
			// then we need to recreate bounds as we may be making the polygon smaller
			bounds.setLocation(x, y);
			bounds.setSize(x, y);
			// add each point to the bounds
			for (int i = 0; i < n; i++) {
				bounds.add(xpoints[i], ypoints[i]);
			}
		} else {
			// the bounds to not contain the point, so simply add it
			bounds.add(x, y);
		}
		// Now we can reset the point!
		xpoints[n] = x;
		ypoints[n] = y;
	} // end modifyPoint()

	/*****************************************************************************************
	* Method used determine the number of verticies this ROI contains.  This number is obtained
	* from the superclass's attribure npoints.
	* @return An integer representing the number of verticies this ROI contains.
	*****************************************************************************************/   
	public int getVertexCount() { return npoints; }
}
