package did.gui;

import java.awt.Cursor;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.KeyListener;

import javax.swing.event.MouseInputListener;

import did.xml.DIDNode;
import did.gui.DIDRoiIFC;
import did.gui.DIDRectRoi;
import did.gui.DIDPolyRoi;
import did.gui.DIDImageCanvas;

/*****************************************************************************************
*<pre>
*File:         DIDCreationHandler.java
*Version:      1.0
*Description:  This class is responsible for the creation of both rectangle and polygon 
*ROIs.  Both types are created using mouse events.  Rectangles are created by pressing the
*mouse at the point where you want the rectangle to begin then dragging the mouse to the 
*proper size.  Releasing the mouse completes the creation process.  Polygons are created 
*by clicking the mouse wherever you want a vertex to be placed.  To complete the Polygon 
*click on the orginal vertex.  
*NOTE:  This class also extends the KeyListener interface but does not implement any special
*code.  Due to an unresolved issue which does not allow KeyEvent detection on any other 
*object than the main Frame all key detection is being handled in the main Frame.   This 
*is only a stopgap measure.  Once the soure of the error has been identified all KeyEvent
*detection should be moved to the appropriate classes.
*
*Modification Log:
*Date          Name    Comment
*xx-xx-xxxx    XXX	   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
*</pre>
******************************************************************************************/

public class DIDCreationHandler implements MouseInputListener, KeyListener {

	/** Part of a group of integers that track and limit mouse movements.  xOrigin and yOrigin
	capture the orignal mouse x, y coordinates(to esatablish a point of origin).  xMax and 
	yMax capture the max horizontal and vertical boundries. x and y are to used to store the
	current mouse position.*/
	private int				xOrigin		= 0,
							yOrigin		= 0,
							xMax		= 0,
							yMax		= 0;
	/**  A reference to the main image component. */
	private DIDImageCanvas	myCanvas	= null;


	/*****************************************************************************************
	* Method used to instantiate the class.  Sets a refernce to the ImageCanvas to which this
	* listener is registered. 
	* @param canvas The ImageCanvas to which this listener is registered. 
	*****************************************************************************************/   
	public DIDCreationHandler(DIDImageCanvas canvas) {
		myCanvas = canvas;
	}

	/*****************************************************************************************
	* Method used to set the maximum x and y coordinates thus establishing a virtual bounding
	* rectangle (defined by the origin and max coordinates) which all ROIs must reside in. 
	* NOTE: In future versions it may be more efficient to construct a Java Rectangle object 
	* and used it's contains() method to determine if an ROI is inside the bounds of the  
	* Image.
	* @param x The largest x coordinate.
	* @param y The largest y coordinate.
	*****************************************************************************************/   
	public void setImageMax(int x, int y) {
		xMax = x;
		yMax = y;
	}

	/*****************************************************************************************
	* Method used to detect mouse clicks.  In this implementation the clicking of a mouse 
	* signifies the fixation of a vertex of a polygon.  If the fixation point is the origin
	* the polygon is complete.  All actions necessary to properly complete the addition of an 
	* ROI are contained in the addROI method.
	* @param e MouseEvent refelecting the current mouse properties.
	* @see DIDPolyRoi#addPoint
	*****************************************************************************************/   
	public void mouseClicked(MouseEvent e) {
		int scaledX = (int)(e.getX()/myCanvas.getScale());
		int scaledY = (int)(e.getY()/myCanvas.getScale());

		if (myCanvas.myMode == DIDImageCanvas.CREATE) {
			switch (myCanvas.activeRoi.getType()) {
				case DIDRoiIFC.POLYGON:
					DIDPolyRoi tmpPoly = (DIDPolyRoi)myCanvas.activeRoi;
					if (tmpPoly.getStatus() == DIDRoiIFC.WAITING) {
						tmpPoly.addPoint(scaledX, scaledY);
						tmpPoly.addPoint(scaledX, scaledY);						
						tmpPoly.setStatus(DIDRoiIFC.CONSTRUCTING);
					} else {
						if ((e.getModifiers()&e.BUTTON3_MASK) != 0) {
							tmpPoly.removeLastPoint();
							tmpPoly.setStatus(DIDRoiIFC.NORMAL);
							addRoi();
							myCanvas.setMode(DIDImageCanvas.EDIT);
							myCanvas.setCursor(Cursor.getDefaultCursor());
						} else {
							tmpPoly.setLastVertex(scaledX, scaledY);
							tmpPoly.addPoint(scaledX, scaledY);
						}
					}
				break;
				case DIDRoiIFC.RECTANGLE:
				break;
			}
		myCanvas.repaint();
		}	
	}
	
	/*****************************************************************************************
	* Method used to respond to the mouse being moved.  In this implementation moving the
	* mouse is relevant to polygon creation.  Once the first vertex of polygon ROI has been 
	* established (mouseClicked) this method take care of drawing a line between the last 
	* vertex to be fixed and the currend mouse position.  This method also ensures the polygon
	* ROI stays within the bounds of the image being edited.  
	* @param e MouseEvent refelecting the current mouse properties.
	* @see #mouseClicked
	* @see DIDPolyRoi#setLastVertex
	*****************************************************************************************/   
	public void mouseMoved(MouseEvent e) {
		int scaledX = (int)(e.getX()/myCanvas.getScale());
		int scaledY = (int)(e.getY()/myCanvas.getScale());

                myCanvas.setMouseCoordinate(scaledX, scaledY);
		if (myCanvas.myMode == DIDImageCanvas.CREATE) {
			switch (myCanvas.activeRoi.getType()) {
				case DIDRoiIFC.POLYGON:
					DIDPolyRoi tmpPoly = (DIDPolyRoi)myCanvas.activeRoi;
					if (tmpPoly.getStatus() == DIDRoiIFC.CONSTRUCTING) {
						int x = scaledX;
						int y = scaledY;
						if (x < 0) { x = 0; }
						if (y < 0) { y = 0; }
						if (x > xMax) { x = xMax; }
						if (y > yMax) { y = yMax; }
						tmpPoly.setLastVertex(x, y);
						myCanvas.repaint();
					}
				break;
				case DIDRoiIFC.RECTANGLE:
				break;
			}
		}
	}

	/*****************************************************************************************
	* Method used to respond to the mouse being pressed.  In this implementation a mouse 
	* pressed siginifies the begining of a rectangle sizing operation.  This method checks to
	* make sure the rectangle's origins are within the bounds of the image being edited.
	* @param e MouseEvent refelecting the current mouse properties.
	*****************************************************************************************/   
	public void mousePressed(MouseEvent e) {
		int scaledX = (int)(e.getX()/myCanvas.getScale());
		int scaledY = (int)(e.getY()/myCanvas.getScale());

		if (myCanvas.myMode == DIDImageCanvas.CREATE) {
			myCanvas.resetFocus();
			switch (myCanvas.activeRoi.getType()) {
				case DIDRoiIFC.RECTANGLE:
					int x = scaledX;
					int y = scaledY;
					if (x < 0) { x = 0; }
					if (y < 0) { y = 0; }
					if (x > xMax) { x = xMax; }
					if (y > yMax) { y = yMax; }
					xOrigin	= x;
					yOrigin	= y;
				break;
				case DIDRoiIFC.POLYGON:
				break;
			}
			myCanvas.repaint();
		}
	}

	/*****************************************************************************************
	* Method used to respond to the user dragging the mouse.  In this implementation, dragging
	* the mouse siginifies the sizing of a newly created rectangle ROI.  This method checks to
	* make sure the rectangle is within the bounds of the image being edited an sets it's
	* size and location accordingly.
	* @param e MouseEvent refelecting the current mouse properties.
	* @see DIDRectRoi#setLocation
	*****************************************************************************************/   
	public void mouseDragged(MouseEvent e) {
		int scaledX = (int)(e.getX()/myCanvas.getScale());
		int scaledY = (int)(e.getY()/myCanvas.getScale());

                myCanvas.setMouseCoordinate(scaledX, scaledY);

		if (myCanvas.myMode == DIDImageCanvas.CREATE) {
			switch (myCanvas.activeRoi.getType()) {
				case DIDRoiIFC.RECTANGLE:
					DIDRectRoi tmpRect = (DIDRectRoi)myCanvas.activeRoi;
					int x = scaledX;
					int y = scaledY;
					if (x < 0) { x = 0; }
					if (y < 0) { y = 0; }
					if (x >= xMax) { x = xMax; }
					if (y >= yMax) { y = yMax; }
					tmpRect.setLocation(Math.min(x, xOrigin), Math.min(y, yOrigin));
					tmpRect.setSize(Math.abs(x - xOrigin), Math.abs(y - yOrigin));
				break;
				case DIDRoiIFC.POLYGON:
				break;
			}
			myCanvas.repaint();
		}
	}

	/*****************************************************************************************
	* Method used to respond to the user releasing the mouse.  In this implementation a mouse
	* release signifies the completion of a Rectangle ROI.  All actions necessary to properly
	* complete the addition of an ROI are contained in the addROI method.
	* @param e MouseEvent refelecting the current mouse properties.
	* @see #addRoi
	*****************************************************************************************/   
	public void mouseReleased(MouseEvent e) {
		int scaledX = (int)(e.getX()/myCanvas.getScale());
		int scaledY = (int)(e.getY()/myCanvas.getScale());

		if (myCanvas.myMode == DIDImageCanvas.CREATE) {
			switch (myCanvas.activeRoi.getType()) {
				case DIDRoiIFC.RECTANGLE:
					myCanvas.activeRoi.setStatus(DIDRoiIFC.NORMAL);
					addRoi();
					myCanvas.setMode(DIDImageCanvas.EDIT);
					myCanvas.setCursor(Cursor.getDefaultCursor());
				break;
				case DIDRoiIFC.POLYGON:
				break;
			}
			myCanvas.repaint();
		}
	}


	/*****************************************************************************************
	* Method used to add an ROI to both the ImageCanvas and TreeModel (XML).  
	*****************************************************************************************/   
	private void addRoi() {
		myCanvas.myROIs.add(myCanvas.activeRoi);
		myCanvas.myRoiChildren.add(myCanvas.activeRoi);
		DIDNode anchorNode = null;
		int vertexCount = myCanvas.activeRoi.getVertexCount();
		DIDNode parentNode = null;
		if (myCanvas.parentRoi != null) {
			parentNode = myCanvas.parentRoi.getAnchorNode();
		}
		anchorNode = myCanvas.myModel.addROI(vertexCount, parentNode,
		       myCanvas.myModel.EntityType.typeToName(myCanvas.myView));
		myCanvas.activeRoi.setAnchorNode(anchorNode);
		anchorNode.setRoi(myCanvas.activeRoi);
		myCanvas.activeRoi.syncData();
	}
	
	/*****************************************************************************************
	* This method is called when the mouse enters the component.  There is no special 
	* implmentation for this application.
	* @param e MouseEvent refelecting the current mouse properties.
	*****************************************************************************************/
	public void mouseEntered(MouseEvent e) { }

	/*****************************************************************************************
	* This method is called when the mouse leaves the component.  There is no special 
	* implmentation for this application.
	* @param e MouseEvent refelecting the current mouse properties.
	*****************************************************************************************/
	public void mouseExited(MouseEvent e) {	}

	/*****************************************************************************************
	* This method is triggered when the user presses a key.  There is no special implmentation
	* for this application.  (See NOTE at top of class)
	* @param e KeyEvent refelecting the current keyboard properties.
	*****************************************************************************************/
	public void keyPressed(KeyEvent e) { } 

	/*****************************************************************************************
	* This method is triggered when the user releases a key.  There is no special implmentation
	* for this application.  (See NOTE at top of class)
	* @param e KeyEvent refelecting the current keyboard properties.
	*****************************************************************************************/
	public void keyReleased(KeyEvent e) { }

	/*****************************************************************************************
	* This method is triggered when the user presses then releases a key.  There is no special
	* implmentation for this application.  (See NOTE at top of class)
	* @param e KeyEvent refelecting the current keyboard properties.
	*****************************************************************************************/
	public void keyTyped(KeyEvent e) { }
}
