package did.gui;

import java.awt.Shape;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.KeyListener;

import java.util.Stack;
import java.util.Vector;
import java.util.Enumeration;

import javax.swing.event.MouseInputListener;

import did.gui.DIDRoiIFC;
import did.gui.DIDImageCanvas;

/******************************************************************************************
*<pre>
*File:         DIDEditHandler.java
*Version:      1.1
*Description:  This class handles all movement and scaling of ROIs (rotation was inteded to 
*be handled by this class but for now is handled by the ImageCanvas).  To do so it must be
registered on the DIDImageCanvas for the type of events that it is to listen for.  This 
*class is setup to handle Mouse and Key events.  Example:
*  DIDEditHandler newEditHandler = new DIDEditHandler(DIDImageCanvas);
*  addMouseMotionListener(newEditHandler)
*Three basic functionalities are exposed by this class: the ablility to translate an ROI to
the active vertex, set the active vertex, and skew the active vertex (Rotation is also 
*performed arount the active vertex but at the moment not by this class).  The active 
*vertex is therefore an important concept in the manipulation of ROIs and is set by holding
*down the control key and clicking on the on the vertex handle(the square around the 
*vertex).  Once set the active vertex appears in green.  
*Note:  This class also extends the KeyListener interface but does not implement any special
*code.  Due to an unresolved issue which does not allow KeyEvent detection on any other 
*object than the main Frame all key detection is being handled in the main Frame.   This 
*is only a stopgap measure.  Once the soure of the error has been identified all KeyEvent
*detection should be moved to the appropriate classes.
*
*Modification Log:
*Date          Name    Comment
*xx-xx-xxxx    XXX	   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
*</pre>
******************************************************************************************/

public class DIDEditHandler implements MouseInputListener, KeyListener {

	/**  A reference to the main image component. */
	private DIDImageCanvas myCanvas = null;

	/*****************************************************************************************
	* Method used to instantiate the class.  Sets a refernce to the ImageCanvas to which this
	* listener is registered. 
	* @param canvas The ImageCanvas to which this listener is registered. 
	*****************************************************************************************/   
	public DIDEditHandler(DIDImageCanvas canvas) {
		myCanvas = canvas;
	}

	/*****************************************************************************************
	* Method used to respond to the mouse being pressed.  In this implementation if the Alt 
	* key is down the active vertex in the Polygon is set to the current mouse position.  If 
	* the Shift key is down then the entire Polygon is translated to the location of the 
	* MouseEvent (Translation is constrained to the active vertex).
	* @param e A MouseEvent class describing the current state of the physical mouse device.
	*****************************************************************************************/
	public void mousePressed(MouseEvent e) {
                int scaledX = (int)(e.getX()/myCanvas.getScale());
                int scaledY = (int)(e.getY()/myCanvas.getScale());

		if (myCanvas.myMode != DIDImageCanvas.CREATE) {
			if (myCanvas.activeRoi != null) {
				myCanvas.activeRoi.finalizeROI();
				if (e.isAltDown()) { myCanvas.activeRoi.setActiveVertexCoord(scaledX, scaledY); }
				else if (e.isShiftDown()) { myCanvas.activeRoi.setTranslation(scaledX, scaledY); }
				myCanvas.activeRoi.syncData();
			}
		}
//		myCanvas.activeRoi.setStatus(DIDRoiIFC.NORMAL);
		myCanvas.repaint();
	}

	/*****************************************************************************************
	* Method used to respond to the mouse being dragged.  In this implementation if the Alt 
	* key is down the active vertex in the Polygon is set to the current mouse position.  If 
	* the Shift key is down then the entire Polygon is translated to the location of the 
	* MouseEvent (Translation is constrained to the active vertex).
	* @param e A MouseEvent class describing the current state of the physical mouse device.
	*****************************************************************************************/   
	public void mouseDragged(MouseEvent e) {
                int scaledX = (int)(e.getX()/myCanvas.getScale());
                int scaledY = (int)(e.getY()/myCanvas.getScale());

		myCanvas.setMouseCoordinate(scaledX, scaledY);

		if (myCanvas.myMode != DIDImageCanvas.CREATE) {
			if (myCanvas.activeRoi != null) {
				myCanvas.activeRoi.finalizeROI();
				if (e.isAltDown()) { myCanvas.activeRoi.setActiveVertexCoord(scaledX, scaledY); }
				else if (e.isShiftDown()) { myCanvas.activeRoi.setTranslation(scaledX, scaledY); }
				myCanvas.activeRoi.syncData();
			}
		}
//		myCanvas.activeRoi.setStatus(DIDRoiIFC.NORMAL);
		myCanvas.repaint();
	}

	/*****************************************************************************************
	* Method used to respond to the mouse being clicked.  In this implementation when the 
	* user clicks on the ImageCanvas without holding down any other keys 
	* @param e A MouseEvent class describing the current state of the physical mouse device.
	*****************************************************************************************/   
	public void mouseClicked(MouseEvent e) {
                int scaledX = (int)(e.getX()/myCanvas.getScale());
                int scaledY = (int)(e.getY()/myCanvas.getScale());

		if (myCanvas.myMode != DIDImageCanvas.CREATE) {
           if (e.isControlDown()) {
		     if (myCanvas.activeRoi != null) {
				myCanvas.activeRoi.finalizeROI();
				if (myCanvas.activeRoi.setActiveVertex(scaledX, scaledY)) { myCanvas.repaint(); }
			 }
		   }
		}
	}

	/*****************************************************************************************
	* Method used to respond to the mouse being released.  There is currently no need to 
	* implement this method in this class.
	* @param e A MouseEvent class describing the current state of the physical mouse device.
	*****************************************************************************************/   
	public void mouseReleased(MouseEvent e) {}

	/*****************************************************************************************
	* Method used to respond to the mouse entering the Canvas.  There is currently no need to 
	* implement this method in this class.
	* @param e A MouseEvent class describing the current state of the physical mouse device.
	*****************************************************************************************/   
	public void mouseEntered(MouseEvent e) {}

	/*****************************************************************************************
	* Method used to respond to the mouse leaving the Canvas.  There is currently no need to 
	* implement this method in this class.
	* @param e A MouseEvent class describing the current state of the physical mouse device.
	*****************************************************************************************/   
	public void mouseExited(MouseEvent e) {}

	/*****************************************************************************************
	* Method used to respond to the mouse moving in the Canvas.  There is currently no need to 
	* implement this method in this class.
	* @param e A MouseEvent class describing the current state of the physical mouse device.
	*****************************************************************************************/   
	public void mouseMoved(MouseEvent e) {
                int scaledX = (int)(e.getX()/myCanvas.getScale());
                int scaledY = (int)(e.getY()/myCanvas.getScale());

		myCanvas.setMouseCoordinate(scaledX, scaledY);
	}


	/*****************************************************************************************
	* This method is triggered when the user presses a key.  There is no special implmentation
	* for this application.  (See note at top of class)
	* @param e KeyEvent refelecting the current keyboard properties.
	*****************************************************************************************/
	public void keyPressed(KeyEvent e) { }

	/*****************************************************************************************
	* This method is triggered when the user releases a key.  There is no special implmentation
	* for this application.  (See note at top of class)
	* @param e KeyEvent refelecting the current keyboard properties.
	*****************************************************************************************/
	public void keyReleased(KeyEvent e) {}

	/*****************************************************************************************
	* This method is triggered when the user presses then releases a key.  There is no special
	* implmentation for this application.  (See note at top of class)
	* @param e KeyEvent refelecting the current keyboard properties.
	*****************************************************************************************/
	public void keyTyped(KeyEvent e) {}
}
