package did.gui;

import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.ColorModel;
import java.awt.image.MemoryImageSource;
import java.awt.image.Raster;
import java.awt.image.renderable.ParameterBlock;
import java.awt.RenderingHints;
import javax.media.jai.RenderedOp;
import javax.media.jai.InterpolationNearest;
import javax.media.jai.InterpolationBilinear;
import javax.media.jai.BorderExtender;
import javax.media.jai.TileCache;
import java.awt.Color;

import java.lang.Boolean;

import java.util.Vector;
import java.util.Enumeration;

import javax.swing.JDesktopPane;
import javax.swing.JComponent;
import javax.swing.Scrollable;
import javax.swing.SwingConstants;

import javax.media.jai.JAI;
import javax.media.jai.TiledImage;
import javax.media.jai.PlanarImage;

import did.xml.DIDNode;
import did.xml.DIDTree;
import did.xml.DIDDefaultTreeModel;
import did.xml.DIDEntityType;

import did.gui.DIDRoiIFC;
import did.gui.DIDPolyRoi;
import did.gui.DIDRectRoi;
import did.gui.DIDEditHandler;
import did.gui.DIDCreationHandler;
import did.gui.DIDSelectionHandler;
import did.gui.DIDFindMsgBox;
import did.gui.DIDInfoPanel;
import did.gui.DIDControlBar;

/******************************************************************************************
*<pre>
*File:         DIDImageCanvas.java
*Version:      1.1
*Description:  This component is an implementation of JComponent that is responsible for the
*display and manipulation of ground truth data (XML).  This is accomplished through a 
*heirarchal series of objects called ROIs (Regions of Interest).  Each ROI has a 
*corresponding node in the DIDDefaultDataModel which it communicates with to retrieve and
*store its vertex data. Note: For future versions of this software it would be wise to 
*investigate turning this class into a viewer in the Swing MVC model.  This would simplify
*the messaging routines(through listeners), make the code much more streamlined and easier 
*to understand and maintain.  Also it would provide a higher degree of abstraction further
*serparating the data form the view and possible allowing other methods for viewing the 
*data using the same data model.
*
*Modification Log:
*Date          Name    Comment
*xx-xx-xxxx    XXX	   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
*</pre>
******************************************************************************************/

public class DIDImageCanvas extends JDesktopPane implements Scrollable {
	/** These Constants define the state of the canvas.  The component is usually in the EDIT state
	unless a new ROI needs to be created in which case it enters the CREATE state.  These could be
	elminated in future versions of the code.*/
	public final static int	EDIT = 0, CREATE = 1, IMAGE_ONLY = 2;
	/** This integer holds the current state of the canvas: Creating OR Editing. */
	public static		int	myMode;
	/** This integer determines what set of ROIs the canvas is to display.  Valid values are 
	contained in the DIDRoiIFC interface (PAGE, ZONE, LINE, CHARACTER).*/
	public static		int myView = DIDEntityType.PAGE;
	/** These constants define the behavior of the vector holding all ROIs.  INIT_CAPACITY sets the 
	initial capacity of the vector while INCREMENT_NUM sets the amount by which the capacity is 
	increased when the vector overflows.*/
	private final static int	INIT_CAPACITY	= 11,
								INCREMENT_NUM	= 3;
	/** Vector whose purpose is to hold and track all the DIDRoiIFC object implementations 
	currently being displayed with the exception of the parent node's ROI.  The ROIs to be displayed
	are dependent on the view setting. */
	protected static		 Vector	myRoiChildren	= new Vector(INIT_CAPACITY, INCREMENT_NUM);
	/** Vector whose purpose is to hold and track all the DIDRoiIFC object implementations of the 
	current data model. */
	protected static		 Vector	myROIs	= null;
	/** A reference to the DIDRoiIFC interface.  This reference is used to point to the ROI that is 
	being manipulated by the user. */
	protected static DIDRoiIFC	activeRoi	= null;
	/** A reference to the DIDRoiIFC interface.  This reference is used to point to the parent ROI of 
	current view.  For example if the view is Page or Zone then the ROI is null but if the view is Line
	then this attribute holds the ROI of the Zone those lines belong to.*/
	protected static DIDRoiIFC	parentRoi	= null;
	/** An integer representing the maximum unit the component should be scrolled (when necessary). */
	protected int maxUnitIncrement = 1;
	/** A double representing the degree and direction by which an ROI should be rotated. (Counter-Clockwise)*/
	protected double myLeftRotationFactor = -.5;
	/** A double representing the degree and direction by which an ROI should be rotated. (Clockwise)*/
	protected double myRightRotationFactor = .5;
	/** JAVA 2D coordinate mapping object necessary to render main image.  Contains any transforms that
	should be applied to the graphics object prior to rendering.*/
	protected AffineTransform	at		= new AffineTransform();
	/** JAI implementation of RenderedImage interface used to display image files.  This class is used 
	becuase the JAI provides support for most TIFF formats (except LZW) and can, via the JAI class,
	construct a PlanarImage from a file.  The PlanarImage can then be used by Java2D's 
	Graphics2D.drawRenderImage() method.*/
	protected Image img = null;
	protected Image scaledImg = null;
	private float	scale = (float)1.0;
	private float	scaleChangeAmount = (float)0.1;
	private int mouseX = 0;
	private int mouseY = 0;
	private DIDFindMsgBox findFrame = null;
	private boolean myArrowDisplay = false;
	protected boolean isSetNext = false;
	/** Reference to a class used to manage the selection of multiple overlapping ROIs. */
	protected DIDSelectionHandler	sHandler	= new DIDSelectionHandler(this);
	/** Reference to a class used to manage the editing of a single ROI. */
	protected DIDEditHandler		eHandler	= new DIDEditHandler(this);
	/** Reference to a class used to manage the creation of new ROIs. */
	protected DIDCreationHandler	cHandler	= new DIDCreationHandler(this);
	/** Reference to the TreeModel for which this class is acting as a viewer. */
	protected DIDDefaultTreeModel	myModel		= null;
	/** Reference to the Tree viewer which is also viewing the data.  The purpose of this attribute is to 
	expose this object to the various handlers so they can manipulate the Tree based on their own set of
	circumstances. */
	protected DIDTree				myTree		= null;
	/**  An information panel to show the entity information briefly. */
	private DIDInfoPanel myInfopanel = null;

	private TrueViz mainFrame;


	/*****************************************************************************************
	* Constructor used to initialize the component.  Sets up listeners, reference to 
	* background image, reference to data model and component state.
	* @param fileName A string representing the absolute path to the image file that this class
	* is to display.
	* @param unit The maximum amount to move the ImageCanvas when scrolling is necessary.
	* @param newTree Reference to DIDTree that is also visualing data (XML).
	* @see #init
	* @see #setImage
	*****************************************************************************************/
	public DIDImageCanvas(String fileName, int unit, DIDTree newTree, TrueViz frame) {
		maxUnitIncrement = unit;
		myTree = newTree;
		mainFrame = frame;
		myModel = (DIDDefaultTreeModel)newTree.getModel();
		myROIs = myModel.getROIs();
		findFrame = new DIDFindMsgBox(this, myModel);

		init();
		setImage(fileName);
	}

	/*****************************************************************************************
	* Method used to initialize this object.  In this implementation this method adds the 
	* appropriate Mouse Listeners (one for Editing, Creating and Selecting ROIs) as well as 
	* initializing the view to the PAGE level and the mode to allow the user to edit.  (Note:
	* The order in which these listeners are added is the order in which they recieve event
	* notification.)
	*****************************************************************************************/
	public void init() {
		addMouseListener(sHandler);
		addMouseListener(eHandler);
		addMouseListener(cHandler);
		addMouseMotionListener(sHandler);
		addMouseMotionListener(eHandler);
		addMouseMotionListener(cHandler);
		addKeyListener(eHandler);
		setView(DIDRoiIFC.PAGE);
		setMode(EDIT);
	}

	/*****************************************************************************************
	* Method used to perform the custom painting necessary to display the appropriate ROIs.
	* The painting is performed in a layered approach.  First the image being Ground Truthed 
	* is painted.  Then based on the ImageCanvas's mode the appropriate ROIs are painted.  For
	* the create mode the parent Roi is painted first then all the children except for the 
	* activeRoi which is painted last to ensure it's on top. For the edit mode if the view is 
	* set to page then all the ROIs are displayed.  If the view is in any other state then 
	* only that set of ROIs is painted along with that sets parent (if applicable).
	* @param g A Graphics object that represents all the visual data contained in this 
	* object's Graphics window.
	*****************************************************************************************/   
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Graphics2D g2d = (Graphics2D)g;
		//DEL g2d.drawRenderedImage(scaledPi,at);
		g2d.drawImage(scaledImg, null, null);

		DIDRoiIFC tmpRoi = null;
		switch (myMode) {
			case CREATE:
				if (activeRoi != null) { 
/*DEL*/					DIDRoiIFC	scaledParentRoi	= null ;
					if (parentRoi != null) { parentRoi.draw(g2d, getScale()); } 
					for (Enumeration children = myRoiChildren.elements() ; children.hasMoreElements() ;) { 
						tmpRoi = (DIDRoiIFC)(children.nextElement());
						if (tmpRoi != activeRoi) { tmpRoi.draw(g2d, getScale()); }
					}
					activeRoi.draw(g2d, getScale()); 
				}
			break;
			case EDIT:
				if (myView == DIDRoiIFC.PAGE) {
					for (Enumeration allROIs = myROIs.elements() ; allROIs.hasMoreElements() ;) { 
						tmpRoi = (DIDRoiIFC)(allROIs.nextElement());
						tmpRoi.draw(g2d, getScale());
					}
				} else {
					if (parentRoi != null) { parentRoi.draw(g2d, getScale()); } 
					for (Enumeration children = myRoiChildren.elements() ; children.hasMoreElements() ;) { 
						tmpRoi = (DIDRoiIFC)(children.nextElement());
						if (tmpRoi != activeRoi) { tmpRoi.draw(g2d, getScale()); }
					}
					if (activeRoi != null) { activeRoi.draw(g2d, getScale()); }
				}
			break;
			case IMAGE_ONLY:
			break;
		}

		//DEL/ displayCoordinate(g2d, mouseX, mouseY);
		mainFrame.controlBar.setMousePos(mouseX, mouseY);
		displayInfopanel();
	}

	/***********************************************************************
	***********************************************************************/
	public void showInfopanel() {
		if (activeRoi != null) {
			if (myInfopanel != null) {
				remove(myInfopanel);
			}
			myInfopanel = new DIDInfoPanel();
			add(myInfopanel);

			DIDNode anchorNode = activeRoi.getAnchorNode();
			String type = anchorNode.getDataType();
			String id = myModel.getAttribute(anchorNode, type+"ID");
			String gt = myModel.getAttribute(anchorNode, "GT_Text");
			myInfopanel.setInfo(type, id, gt);
			Rectangle bounds = activeRoi.getBounds();
			int x = (int)((bounds.getX()+bounds.getWidth())*scale);
			int y = (int)(bounds.getY()*scale);
		        myInfopanel.show(x, y);
		}
	}

	/***********************************************************************
	***********************************************************************/
	public void displayInfopanel() {

		if (activeRoi == null && myInfopanel != null) {
			myInfopanel.setVisible(false);
			remove(myInfopanel);
			myInfopanel = null;
		}
	}



	/*****************************************************************************************
	* This method is responsible for creating an image object from a file reference.  The 
	* image object is basically a background on top of which all ROIs are drawn.  The image
	* object is created using a call to the JAI class's create method.  This method reads the
	* filename passed to it and returns a PlanarImage which is an implementation of the 
	* RenderedImage interface.  The PalnarImage can then be painted on this object using the 
	* drawRenderedImage() method of the Graphics2D object in the paintComponent() method.
	* @param e String reprsenting the filename from which an image object should be created.
	* @see #paintComponent
	*****************************************************************************************/
	public void setImage(String fileName) {
		PlanarImage pi = null;
		PlanarImage scaledPi = null;

		try	{
			scaledPi = pi = JAI.create("fileload", fileName);
			setScale((float)1.0);
		} catch (IllegalArgumentException e) {
			scaledPi = JAI.create("fileload", "pics//splash.tif");
		}

		//---- Convert PlanarImage to Image
		if (scaledPi.getColorModel().getPixelSize() == 1) {
			Raster newRaster = null;
			if (scaledPi != null) newRaster = scaledPi.getData();

			int[] iarray = null;
			int[] idata = newRaster.getPixels(0, 0,
				newRaster.getWidth(),
				newRaster.getHeight(), iarray);
			ColorModel cm = scaledPi.getColorModel();
			int[] val = {(255<<24)|(cm.getRed(0)<<16)|
					(cm.getGreen(0)<<8)|cm.getBlue(0),
				     (255<<24)|(cm.getRed(1)<<16)|
					(cm.getGreen(1)<<8)|cm.getBlue(1)};

			for (int i=0; i< newRaster.getHeight(); i++) {
				for (int j=0; j< newRaster.getWidth(); j++) {
					int index = i*newRaster.getWidth() + j;
					idata[index] = val[idata[index]];
				}
			}

			scaledImg = img = createImage(new MemoryImageSource(
				newRaster.getWidth(),
				newRaster.getHeight(),
				idata, 0, newRaster.getWidth()));
		} else  {
			scaledImg = img = (Image)scaledPi.getAsBufferedImage();
		}

		if (pi == null)  img = null;

		setPreferredSize(new Dimension(scaledImg.getWidth(null),
					scaledImg.getHeight(null)));
		cHandler.setImageMax(scaledImg.getWidth(null),
					scaledImg.getHeight(null));
		//DEL repaint();
		revalidate();
	}


	/**************************************************************************
        *
	**************************************************************************/
	public void setScale(float newscale) {
		if (newscale > 0) {
			scale = newscale;
		}
	}

	/**************************************************************************
        *
	**************************************************************************/
	public float getScale() {
		return scale;
	}

	/**************************************************************************
        *
	**************************************************************************/
	public void zoomIn() {
		changeScale(scale + scaleChangeAmount);
	}

	/**************************************************************************
        *
	**************************************************************************/
	public void zoomOut() {
		changeScale(scale - scaleChangeAmount);
	}


	/**************************************************************************
        *
	**************************************************************************/
	public void changeScale(float newscale) {
		if(img == null) {
			return;
		}

		setScale(newscale);

		setSize(new Dimension((int)(scale*img.getWidth(null)),
				(int)(scale*img.getHeight(null))));
                scaledImg = img.getScaledInstance(
				(int)(scale*img.getWidth(null)),
				(int)(scale*img.getHeight(null)),
				Image.SCALE_DEFAULT);
		setPreferredSize(new Dimension(scaledImg.getWidth(null),
					       scaledImg.getHeight(null)));
		revalidate();
		showInfopanel();

	}


	/*****************************************************************************************
	* Method used to create new ROIs.  Based on the parameter passed to this method, the 
	* appropriate type of RIO is created.  This method also sets the showCoordinates attribute
	* for the ROI being created based on the current view and the corresponding setting the 
	* trueviz.properties file. 
	* @param type An integer representing the type of ROI to create.  This integer comes from
	* the DIDRoiIFC interface constants used to define the type of ROI.
	*****************************************************************************************/   
	public void newRoi(int type) {
		setMode(CREATE);
		setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
		resetFocus(); 
		switch (type) {
			case DIDRoiIFC.RECTANGLE:
				activeRoi = (DIDRoiIFC)(new DIDRectRoi());
			break;
			case DIDRoiIFC.POLYGON:
				activeRoi = (DIDRoiIFC)(new DIDPolyRoi());
			break;
		}
		DIDProperties newProps = DIDProperties.getInstance();
		Boolean showCoords = null;

		String typeName;
		if ((typeName = myModel.EntityType.typeToName(myView)) != null) {
			showCoords = new Boolean(newProps.getProperty(typeName + "Coordinates"));
			activeRoi.setCoordDisp(showCoords.booleanValue());
		}
	}

	/*****************************************************************************************
	* Method used to remove and ROI and its corresponding data from the display and data file
	* respectively.  This process involves deleting the 
	* NOTE: Explore using myModel.removeNodeFromParent(childNode) instead of reload to delete.
	* The problem with using reload is that even though it does meassage all TreeModelListners
	* registered on this model is does not fire the NodesRemoved method.  Instead it fires the
	* TreeStructureChanged (which makes sense because you could be deleting a large protion of 
	* the tree).
	*****************************************************************************************/
	public void deleteRoi() {
		if (activeRoi != null) {
			DIDNode child = activeRoi.getAnchorNode();
			DIDNode parent = (DIDNode)child.getParent();
			/** This method is effective in removing all the ROIs below the active one but 
			it is not terribly efficeint and should be replaced in future verions. */
			for (Enumeration roi = child.breadthFirstEnumeration(); roi.hasMoreElements(); ) {
				DIDNode node = (DIDNode)roi.nextElement();
				DIDRoiIFC roiToDelete = node.getRoi();
				if (roiToDelete != null) { myROIs.remove(roiToDelete); }
			}
			myROIs.remove(activeRoi);
			myRoiChildren.remove(activeRoi);
			child.deleteDomNode();
			child.removeFromParent();
			myModel.reload(parent);
			sHandler.resetSelection();
			repaint();
		}
	}

	/*****************************************************************************************
	* Method used to set the Mode of the ImageCanvas.  The mode is used to dertermine what 
	* this object should paint.  If the user is creating ROIs then it should follow one 
	* painting path and if the user is editing ROIs then it should follow another path.  This
	* methodology was usefull in early versions of the code but now is somewhat antiquated 
	* and could be eliminated in future versions.
	* @param An integer used to determine the mode of the object. 
	* @see #paintComponent
	*****************************************************************************************/   
	public void setMode(int mode) {
		myMode = mode;
		repaint();
	}

	/*****************************************************************************************
	* Method used to get the Mode of the ImageCanvas.  This method is also somewhat antiquated
	* and could be eliminated in future verions.
	* @return An integer representing the mode of the ImageCanvas (Create, Edit).
	*****************************************************************************************/   
	public int getMode() { return myMode; }

	/*****************************************************************************************
	* Method used to determine which level of ROIs the ImageCanvas should display.  There are 
	* currently four level: Page, Zone, Line, and Character.  To pass from one level to 
	* another there a series of checks to ensure that there is an ROI selected (else how do
	* you know what sub-level ROIs to display) and the proper progression of level is being
	* used.  This method also sets the parentRoi based on the level and constructs the 
	* myRoiChildren.
	* @param newView An interger constant from the DIDRoiIFC class that represent the level 
	* of ROIs to display.
	* @return True if the view changed to the setting passed in as a parameter, false if it 
	* did not change.
	*****************************************************************************************/   
	public boolean setView(int newView) {
		boolean viewChanged = false;
		//DEL if (newView != myView) {
			if (newView == DIDRoiIFC.PAGE) {
				myView = newView;
				viewChanged = true;
			} else {
				if (myModel.EntityType.ancestor(myView, newView)) {
					parentRoi = activeRoi;
				} else {
					parentRoi = null;
				}
				myView = newView;
				myRoiChildren.clear();
				for (Enumeration rois = myROIs.elements(); rois.hasMoreElements(); ) {
					DIDRoiIFC tmpRoi = (DIDRoiIFC)rois.nextElement();
					if ( tmpRoi.getNodeType() == myView) {
						myRoiChildren.add(tmpRoi);
					}
				}
				viewChanged = true;
			}
			sHandler.resetSelection();
			repaint();
		//DEL }
		return (viewChanged);
	}

	/*****************************************************************************************
	* Method used to return the current view of this ImageCanvas (Page, Zone, Line, Character).
	* @return An integer representing the current view the 
	*****************************************************************************************/   
	public int getView() { return myView; }

	/*****************************************************************************************
	* Method used to clear all the zones from the screen.  Clears the vector that holds all 
	* ROI objects, the vector that holds ROI children and reset the Mulit-select mechanism.
	*****************************************************************************************/   
	public void reset() {
		myROIs.clear();
		myRoiChildren.clear();
		sHandler.resetSelection();
	}

	/*****************************************************************************************
	* Method used take the focus away from all the ROIs on the screen (mainly to releave 
	* formatting issues).  
	*****************************************************************************************/   
	public void resetFocus() {
		DIDRoiIFC tmpRoi = null;
		for (Enumeration e = myROIs.elements(); e.hasMoreElements() ; ) {
			tmpRoi = (DIDRoiIFC)e.nextElement();
			tmpRoi.setFocus(false);
			tmpRoi.setSelected(false);
		}
	}
	
	/*****************************************************************************************
	* Method used rotate the selected ROI to the left.  This method does so by setting the 
	* rotation factor of the selected ROI and then repainting the entire canvas. 
	* NOTE: The rotation is performed around the active vertex.  To finalize the rotation the
	* user must click on the ROI after rotation is complete.
	*****************************************************************************************/   
	public void rotateLeft() {
		if (activeRoi != null) { 
			activeRoi.rotate(myLeftRotationFactor); 
			activeRoi.syncData();
		}
		repaint();
	}

	/*****************************************************************************************
	* Method used rotate the selected ROI to the right.  This method does so by setting the
	* rotation factor of the selected ROI and then repainting the entire canvas.
	* NOTE: The rotation is performed around the active vertex.  To finalize the rotation the
	* user must click on the ROI after rotation is complete.
	*****************************************************************************************/   
	public void rotateRight() {
		if (activeRoi != null) { 
			activeRoi.rotate(myRightRotationFactor);
			activeRoi.syncData();
		};
		repaint();
	}

	/*****************************************************************************************
	* Part of Scrollable Interface Implementation used to return the preferred display size 
	* of the ImageCanvas.
	* @return Dimension This method returns a Dimension object represention the perferred size
	* of the this object.
	*****************************************************************************************/
	public Dimension getPreferredScrollableViewportSize() { 
		return getPreferredSize();
	}

	/*****************************************************************************************
	* Part of Scrollable Interface Implementation necessary to determine the amount by which 
	* to scroll the object in the viewport when necessary.
	* @return This method returns an integer represeting the number of pixels to scroll.
	* @param visibleRect A Rectangle object representing the visible portion of the component.
	* @param orientation Either SwingConstants.VERTICAL or SwingConstants.HORIZONTAL
	* @param direction Less than zero to scroll up/left, greater than zero for down/right.
	*****************************************************************************************/
	public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation, int direction) {
		int currentPosition = 0;
		if (orientation == SwingConstants.HORIZONTAL) { 
			currentPosition = visibleRect.x;
		} else {
			currentPosition = visibleRect.y;
		}
		if (direction < 0) {
			int newPosition = currentPosition - 
				(currentPosition / maxUnitIncrement) * maxUnitIncrement;
			return (newPosition == 0) ? maxUnitIncrement : newPosition;
		} else { 
			return ((currentPosition / maxUnitIncrement) + 1) *
				maxUnitIncrement - currentPosition;
		}
	}
	
	/*****************************************************************************************
	* Method used to determine the amount for a block inrecment, which is the height or width 
	* of visibleRect, based on orientation.
	* @param visibleRect The view area visible within the viewport 
	* @param orientation Either SwingConstants.VERTICAL or SwingConstants.HORIZONTAL
	* @param direction Less than zero to scroll up/left, greater than zero for down/right.
	* @return Returns the amount for a block inrecment, which is the height or width of 
	* visibleRect, based on orientation
	*****************************************************************************************/   
	public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation, int direction) { 
		if (orientation == SwingConstants.HORIZONTAL) { 
			return visibleRect.width - maxUnitIncrement;
		} else {
			return visibleRect.height - maxUnitIncrement;
		}
	}

	/*****************************************************************************************
	* Returns false to indicate that the height of the viewport does not determine the height 
	* of the table, unless the preferred height of the tree is smaller than the viewports 
	* height. In other words: ensure that the tree is never smaller than its viewport.
	* @return false
	*****************************************************************************************/   
	public boolean getScrollableTracksViewportHeight() {
		return false;
	}
	
	/*****************************************************************************************
	* Returns false to indicate that the width of the viewport does not determine the width of
	* the table, unless the preferred width of the tree is smaller than the viewports width. 
	* In other words: ensure that the tree is never smaller than its viewport.
	* @return false
	*****************************************************************************************/   
	public boolean getScrollableTracksViewportWidth() {
		return false;
	}

	/*****************************************************************************************
	* Method used to set the maximum unit (in pixels) by which this object should scroll.  
	* @param pixels An integer representing the number of pixels to scroll by when scrolling
	* is called necessary.
	*****************************************************************************************/   
	public void setMaxUnitIncrement(int pixels) {
		maxUnitIncrement = pixels;
	}

	/*********************************************************************
	*********************************************************************/
	public void setMouseCoordinate(int x, int y) {
		mouseX = x;
		mouseY = y;
		repaint();
	}

/*DEL====>
	public void displayCoordinate(Graphics2D g2d, int x, int y) {
		int coordDisplayX = 10;
		int coordDisplayY = 10;
		String myString = new String(x + ", " + y);
		g2d.setColor(Color.black);
		g2d.drawString(myString, coordDisplayX, coordDisplayY);
	}
<====DEL*/

	/*********************************************************************
	*********************************************************************/
	public void searchFind() {
		findFrame.show();
	}

	/*********************************************************************
	*********************************************************************/
	public void searchFindDir() {
	}

	/*********************************************************************
	*********************************************************************/
	public boolean arrowDisplay() {
		return myArrowDisplay;
	}

	/*********************************************************************
	*********************************************************************/
	public void setArrowDisplay(boolean boolVal) {
		myArrowDisplay = boolVal;
	}

	/*********************************************************************
	*********************************************************************/
	public void setArrowDisplay(int entityType, boolean boolVal) {
		String entityTypeName = myModel.EntityType.typeToName(entityType);
		Enumeration rois = myROIs.elements();

		if (entityTypeName == null) {
			while (rois.hasMoreElements()) {
				DIDRoiIFC tmpRoi = (DIDRoiIFC)rois.nextElement();
				if(boolVal) {
					DIDProperties newProps = DIDProperties.getInstance();
					DIDNode node = tmpRoi.getAnchorNode();
					Boolean showArrow = new Boolean(newProps.getProperty(node.getNodeName() + "Arrow"));
					tmpRoi.setDrawArrow(showArrow.booleanValue());
				} else {
					tmpRoi.setDrawArrow(boolVal);
				}
			}
		} else {
			while (rois.hasMoreElements()) {
				DIDRoiIFC tmpRoi = (DIDRoiIFC)rois.nextElement();
				if (tmpRoi.getNodeType() == entityType) {
					tmpRoi.setDrawArrow(boolVal);
				}
			}
		}
	}


	/*********************************************************************
	*********************************************************************/
	public void setFillColors(boolean fill) {
		if (activeRoi != null) {
			activeRoi.setFocus(false);
		}
		Enumeration rois = myROIs.elements();
		while (rois.hasMoreElements()) {
			DIDRoiIFC tmpRoi = (DIDRoiIFC)rois.nextElement();
			if(fill && !tmpRoi.hasFocus() &&
					(myView == DIDEntityType.PAGE ||
					myView == tmpRoi.getNodeType())) {
				tmpRoi.setFillColor(tmpRoi.getNoFocusColor());
			} else if (!tmpRoi.hasFocus()) {
				tmpRoi.setFillColor(null);
			}
		}
		if (activeRoi != null) {
			activeRoi.setFocus(true);
		}
	}


	/*********************************************************************
	*********************************************************************/
	public void setNext() {
		if (activeRoi != null) {
			isSetNext = true;
		}
	}

	/*********************************************************************
	*********************************************************************/
	public void setNext(DIDRoiIFC nextRoi) {
		DIDNode anchorNode = null;
		if (activeRoi != null) {
			anchorNode = activeRoi.getAnchorNode();
			activeRoi.setNextRoi((DIDBaseRoi)nextRoi);
		}
		DIDNode nextNode = null;
		if (nextRoi != null) {
			nextNode = nextRoi.getAnchorNode();
		}
		if (anchorNode != null && nextNode != null) {
			String type = nextNode.getDataType();
			String nextID = myModel.getAttribute(nextNode, type+"ID");
			String nodeType = anchorNode.getDataType();
			Enumeration e = anchorNode.breadthFirstEnumeration();
			while (e.hasMoreElements()) {
				DIDNode tmpNode = (DIDNode)e.nextElement();
				if (tmpNode.getNodeName().equals(nodeType+"Next")) {
					Enumeration attr = tmpNode.children();
					while (attr.hasMoreElements()) {
						DIDNode tmpAttr = (DIDNode)attr.nextElement();
						if (tmpAttr.getNodeName().equals("Value")) {
                                                        tmpAttr.setNodeValue(nextID);
							break;
                                                }
                                        }
					break;
                                }
                        }
                }
	}

	/*********************************************************************
	*********************************************************************/
	public void deleteNext() {
		DIDNode anchorNode = null;
		if (activeRoi != null) {
			anchorNode = activeRoi.getAnchorNode();
			activeRoi.setNextRoi(null);
		}
		if (anchorNode != null) {
			String nodeType = anchorNode.getDataType();
			Enumeration e = anchorNode.breadthFirstEnumeration();
			while (e.hasMoreElements()) {
				DIDNode tmpNode = (DIDNode)e.nextElement();
				if (tmpNode.getNodeName().equals(nodeType+"Next")) {
					Enumeration attr = tmpNode.children();
					while (attr.hasMoreElements()) {
						DIDNode tmpAttr = (DIDNode)attr.nextElement();
						if (tmpAttr.getNodeName().equals("Value")) {
                                                        tmpAttr.setNodeValue(""); // set to nil
							break;
                                                }
                                        }
					break;
                                }
                        }
                }
	}

}

