package did.gui;

import java.awt.Color;
import java.awt.Polygon;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;

import did.gui.DIDBaseRoi;
import did.xml.DIDNode;

/*****************************************************************************************
*<pre>
*File:         DIDPolyROI.java
*Version:      1.1
*Description:  This class is an extension of the DIDBaseRoi class and represents a polygon.
*It can have any number of verticies and is not constrained to any particular shape during
*its creation.  The draw() method handles all custom rendering for this class including the 
*different look a polygon has as its being created as apposed to when its being edited.
*
*Modification Log:
*Date          Name    Comment
*xx-xx-xxxx    XXX	   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
*</pre>
*@see DIDCreationHandler#mouseClicked
*@see DIDCreationHandler#mouseMoved
******************************************************************************************/

public class DIDPolyRoi extends DIDBaseRoi {

	/*****************************************************************************************
	* Constructor used to initalize the object.  Initializes the ROI from the specified 
	* parameters.
	* @param tmpX An array of integers corresponding to all the x corrdinates for this ROI.
	* @param tmpY An array of integers corresponding to all the y corrdinates for this ROI.
	* @param tmpN The number of verticies this ROI should contain.
	*****************************************************************************************/   
	public DIDPolyRoi(int[] x, int[] y, int n) { super(x, y, n); }

	/*****************************************************************************************
	* Method used to instantiate this class.
	*****************************************************************************************/   
	public DIDPolyRoi() { super(); }

	/*****************************************************************************************
	* Method used to render this object onto an existing graphics object passed in as a
	* parameter.  This particular implementation is responsible for rendering the vertex 
	* handles, vertex coordinates and drawing the polygon itself based on the status.
	* @param g2d A reference to a specific Graphics2D object onto which this ROI should render
	* itself.
	* @see DIDBaseRoi#at
	* @see DIDBaseRoi#pi
	*****************************************************************************************/   
	public void draw(Graphics2D g2d) {
		int i = 0;
		int[] x = new int[npoints + 1];
		int[] y = new int[npoints + 1];
		g2d.setFont(myFont);
		pi = this.getPathIterator(at);
		while (pi.isDone() != true) {
			double myArray[] = new double[6];
			if (i == myActiveVertex) { g2d.setColor(myActiveVertexColor); }
			else { g2d.setColor(myVertexColor); }
			int mySegType = pi.currentSegment(myArray);
			switch (mySegType) {
					case pi.SEG_MOVETO : case pi.SEG_LINETO :
						if (focus) {
							g2d.fillRect((int)myArray[0] + X_OFFSET, (int)myArray[1] + Y_OFFSET,
								CORNER_WIDTH, CORNER_HEIGHT);
						}
						x[i] = (int)myArray[0];
						y[i] = (int)myArray[1];
					break;
					default :
					break;
			}
			if (showCoordinates && focus) {
				String myString = new String((int)myArray[0] + ", " + (int)myArray[1]);
				g2d.setColor(myFontColor);
				g2d.drawString(myString, (int)myArray[0], (int)myArray[1]);				
			}
			i++;
			pi.next();
		}
		if (focus) { g2d.setColor(myFocusColor); }
		else { g2d.setColor(myNoFocusColor); }
		if (myStatus != CONSTRUCTING) {
			x[npoints] = x[0];
			y[npoints] = y[0];
			g2d.drawPolyline(x, y, npoints + 1);
		} else {
			g2d.drawPolyline(x, y, npoints);
		}		
	}

	public void draw(Graphics2D g2d, float scale) {
		int i = 0;
		int[] x = new int[npoints + 1];
		int[] y = new int[npoints + 1];

		g2d.setFont(myFont);
		pi = this.getPathIterator(at.getScaleInstance(scale, scale));
		while (pi.isDone() != true) {
			double myArray[] = new double[6];
			if (i == myActiveVertex) { g2d.setColor(myActiveVertexColor); }
			else { g2d.setColor(myVertexColor); }
			int mySegType = pi.currentSegment(myArray);
			switch (mySegType) {
					case pi.SEG_MOVETO : case pi.SEG_LINETO :
						if (focus) {
							g2d.fillRect((int)(myArray[0] + X_OFFSET),
								(int)(myArray[1] + Y_OFFSET),
								(int)(CORNER_WIDTH),
								(int)(CORNER_HEIGHT));
						}
						x[i] = (int)(myArray[0]);
						y[i] = (int)(myArray[1]);
					break;
					default :
					break;
			}
			if (showCoordinates && focus) {
				String myString = new String((int)(myArray[0]/scale) + ", " + (int)(myArray[1]/scale));
				g2d.setColor(myFontColor);
				g2d.drawString(myString, (int)(myArray[0]), (int)(myArray[1]));				
			}
			i++;
			pi.next();
		}
		if (focus) { g2d.setColor(myFocusColor); }
		else if(selected) { g2d.setColor(Color.orange); }
		else { g2d.setColor(myNoFocusColor); }
		if (myStatus != CONSTRUCTING) {
			x[npoints] = x[0];
			y[npoints] = y[0];

			if (myFillColor != null)  {
				Color baseColor = Color.white;
				DIDNode tmpNode = (DIDNode)getAnchorNode().getParent();
				DIDRoiIFC tmpRoi;
				if (tmpNode != null && (tmpRoi = tmpNode.getRoi()) != null) {
					if (tmpRoi.getFillColor() != null) {
						baseColor = tmpRoi.getFillColor();
					}
				}
				g2d.setXORMode(baseColor);
				g2d.setColor(myFillColor);
				g2d.fillPolygon(x, y, npoints + 1);
			}

			g2d.setPaintMode();
			g2d.setColor(myNoFocusColor);
			g2d.drawPolyline(x, y, npoints + 1);
		} else {
			g2d.drawPolyline(x, y, npoints);
		}		
		if(myDrawArrow) {
			drawArrow(g2d, scale);
		}
	}
	/*****************************************************************************************
	* Method used to set the x and y values of the last vertex in the base class coordinate
	* arrays (xpoints, ypoints).  This method is used mainly in the polygon creation process.
	* @param newX New x coordinate.
	* @param newY New y coordinate.
	*****************************************************************************************/   
	public void setLastVertex(int newX, int newY) {
		modifyPoint(npoints - 1, newX, newY);
	}

	/*****************************************************************************************
	* Method used to remove the last vertex in the ROI's x & y coordinate arrays.  This method
	* is specific to the Polygon ROI implementation.  When creating a polygon ROI it is 
	* necessary to always have one extra vertex for rendering purposes.  Upon confirmation of 
	* completion this last vertex is no longer needed and should be remove using this method.  
	* @see DIDCreationHandler#mouseClicked
	*****************************************************************************************/   
	public void removeLastPoint() {
		npoints = npoints - 1;
		modifyPoint(npoints, 0, 0);
	}

	/*****************************************************************************************
	* Method used to determine what type of ROI this class implments (Rectangle, Polygon).
	* Use RECTANGLE and POLYGON attributes in DIDRoiIFC interface.  
	* @return Integer representing the type of ROI (Rectangle or Polygon).
	*****************************************************************************************/   
	public int getType() { return DIDRoiIFC.POLYGON; }
}
