package did.gui;

import java.awt.Font;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;

import did.gui.DIDBaseRoi;
import did.xml.DIDNode;

/*****************************************************************************************
*<pre>
*File:         DIDRectROI.java
*Version:      1.1
*Description:  This class is an extension of the DIDBaseRoi class and represents a rectangle.
*It can only have four verticies and is constrained to a rectangular shape during its 
*creation.  The draw() method handles all custom rendering for this class.  The constraint 
*of this ROI to a rectangular shape is accomplished through the use of the setSize() and 
*setLocation() methods. 
*NOTE: This ROI is only guaranteed to retain its rectangular shape during the initial 
*creation after which it may be skewed or rotated to any shape.
*
*Modification Log:
*Date          Name    Comment
*xx-xx-xxxx    XXX	   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
*</pre>
******************************************************************************************/

public class DIDRectRoi extends DIDBaseRoi {

	/*****************************************************************************************
	* Method used to instantiate this class.  Makes a call to the superclass telling it to 
	* create four verticies since this is a rectangle.
	*****************************************************************************************/   
	public DIDRectRoi() { super(4); }

	/*****************************************************************************************
	* Constructor used to initalize the object.  Initializes the ROI from the specified 
	* parameters.
	* @param tmpX An array of integers corresponding to all the x corrdinates for this ROI.
	* @param tmpY An array of integers corresponding to all the y corrdinates for this ROI.
	* @param tmpN The number of verticies this ROI should contain.
	*****************************************************************************************/   
	public DIDRectRoi(int[] x, int[] y, int n) { super(x, y, n); }

	/*****************************************************************************************
	* Method used to render this object onto an existing graphics object passed in as a
	* parameter.  This particular implementation is responsible for rendering the vertex 
	* handles, vertex coordinates and drawing the rectangle itself.
	* @param g2d A reference to a specific Graphics2D object onto which this ROI should render
	* itself.
	* @see DIDBaseRoi#at
	* @see DIDBaseRoi#pi
	*****************************************************************************************/   
	public void draw(Graphics2D g2d) {
		int i = 0;
		int[] x = new int[npoints + 1];
		int[] y = new int[npoints + 1];
		g2d.setFont(myFont);
		pi = this.getPathIterator(at);
		while (pi.isDone() != true) {
			double myArray[] = new double[6];
			if (i == myActiveVertex) { g2d.setColor(myActiveVertexColor); }
			else { g2d.setColor(myVertexColor); }
			int mySegType = pi.currentSegment(myArray);
			switch (mySegType) {
					case pi.SEG_MOVETO : case pi.SEG_LINETO :
						if (focus) {
							g2d.fillRect((int)myArray[0] + X_OFFSET, (int)myArray[1] + Y_OFFSET,
								CORNER_WIDTH, CORNER_HEIGHT);
						}
						x[i] = (int)myArray[0];
						y[i] = (int)myArray[1];
					break;
					default :
					break;
			}
			if (showCoordinates && focus) {
				String myString = new String((int)myArray[0] + ", " + (int)myArray[1]);
				g2d.setColor(myFontColor);
				g2d.drawString(myString, (int)myArray[0], (int)myArray[1]);				
			}
			i++;
			pi.next();
		}
		if (focus) { g2d.setColor(myFocusColor); }
		else { g2d.setColor(myNoFocusColor); }
		x[npoints] = x[0];
		y[npoints] = y[0];
		g2d.drawPolyline(x, y, npoints + 1);
	}

	public void draw(Graphics2D g2d, float scale) {
		int i = 0;
		int[] x = new int[npoints + 1];
		int[] y = new int[npoints + 1];

		g2d.setFont(myFont);
		pi = this.getPathIterator(at.getScaleInstance(scale, scale));
		while (pi.isDone() != true) {
			double myArray[] = new double[6];
			if (i == myActiveVertex) { g2d.setColor(myActiveVertexColor); }
			else { g2d.setColor(myVertexColor); }
			int mySegType = pi.currentSegment(myArray);
			switch (mySegType) {
					case pi.SEG_MOVETO : case pi.SEG_LINETO :
						if (focus) {
							g2d.fillRect((int)myArray[0] + X_OFFSET, (int)myArray[1] + Y_OFFSET,
								CORNER_WIDTH, CORNER_HEIGHT);
						}
						x[i] = (int)(myArray[0]);
						y[i] = (int)(myArray[1]);
					break;
					default :
					break;
			}
			if (showCoordinates && focus) {
				String myString = new String((int)(myArray[0]/scale) + ", " + (int)(myArray[1]/scale));
				g2d.setColor(myFontColor);
				g2d.drawString(myString, (int)myArray[0], (int)myArray[1]);				
			}
			i++;
			pi.next();
		}
		if (focus) { g2d.setColor(myFocusColor); }
		else if(selected) { g2d.setColor(Color.orange); }
		else { g2d.setColor(myNoFocusColor); }
		x[npoints] = x[0];
		y[npoints] = y[0];
//		if(myStatus != CONSTRUCTING) {
		if(myFillColor != null ) {
			if (myFillColor != null)  {
				Color baseColor = Color.white;
				DIDNode tmpNode = (DIDNode)getAnchorNode().getParent();
				DIDRoiIFC tmpRoi;
				if (tmpNode != null && (tmpRoi = tmpNode.getRoi()) != null) {
					if (tmpRoi.getFillColor() != null) {
						baseColor = tmpRoi.getFillColor();
					}
				}
				g2d.setXORMode(baseColor);
				g2d.setColor(myFillColor);
				g2d.fillPolygon(x, y, npoints + 1);
			}

			g2d.setPaintMode();
			g2d.setColor(myNoFocusColor);
			g2d.drawPolyline(x, y, npoints + 1);
		}
		else {
			g2d.drawPolyline(x, y, npoints + 1);
		}
		if(myDrawArrow) {
			drawArrow(g2d, scale);
		}
	}

	/*****************************************************************************************
	* Method used to set the location of the ROI.  Translation is performed on the first 
	* element in each coordinate array (xpoints, ypoints).
	* @param x New x location.
	* @param y New y loaction.
	*****************************************************************************************/   
	public void setLocation(int x, int y) {
		translate(x - xpoints[0], y - ypoints[0]);
	}

	/*****************************************************************************************
	* Method used to set the size of the ROI.  When used in conjunction with the setLocation 
	* method both functions help enforce the rectangular shape of this ROI.
	* @param w The new width of the ROI.
	* @param h The new height of the ROI. 
	* @see #setLocation
	*****************************************************************************************/   
	public void setSize(int w, int h) {
		modifyPoint(1, xpoints[0] + w, ypoints[0]);
		modifyPoint(2, xpoints[0] + w, ypoints[0] + h);
		modifyPoint(3, xpoints[0], ypoints[0] + h);
	}

	/*****************************************************************************************
	* Method used to get the type (Rectangle, Polygon) of a particular implementation of this 
	* class.
	* @return An integer repesenting the type of ROI (Rectangle, Polygon).
	*****************************************************************************************/   
	public int getType() { return DIDRoiIFC.RECTANGLE; }
}
