package did.gui;

import java.io.Serializable;

import java.awt.Shape;
import java.awt.Color;
import java.awt.Graphics2D;

import did.xml.DIDNode;

/*****************************************************************************************
*<pre>
*File:         DIDRoiIFC.java
*Version:      1.1
*Description:  This interface is key to the functionality of this application as is it 
*encapsulates all the common functionality necessary for the ROI architecture to work
*properly.  When implemented it allows ROIs (Rectangles, Polygons and any future 
*implementation) to be passed around and used without knowledge of how the underlying 
*methods are implemented.
*
*Modification Log:
*Date          Name    Comment
*xx-xx-xxxx    XXX	   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
*</pre>
******************************************************************************************/

public interface DIDRoiIFC extends Shape, Serializable {
	
	/** Constant(s) denoting the status of the ROI.*/
	public	static	final	int WAITING = -1, CONSTRUCTING = 0, MOVING = 1,	RESIZING = 2, ROTATING = 3, NORMAL = 4;
	/** Constant(s) denoting the kind of ROI.  This variable is used by the ImageCanvas to distinguish between
	Rectangles, Polygons and any future kinds of RIOs.*/
	public	static	final	int	BASE = 0, RECTANGLE = 1, POLYGON = 2;
	/** Constant(s) denoting the type of data the ROI represents.*/
	public static	final	int UNDETERMINED = -2, UNSPECIFIED = -1, PAGE = 0, ZONE = 1, LINE = 2, WORD = 3, CHARACTER = 4;

	/*****************************************************************************************
	* This method renders the object on the screen.  Not only does it draw the ROI itself but 
	* also coordinate information and vertex handles.
	* @param g A reference to a specific graphics object.  In the absence of a reference this
	* parameter refers to the current object.
	*****************************************************************************************/   
	public void draw (Graphics2D g2d);
	public void draw (Graphics2D g2d, float scale);

	/*****************************************************************************************
	* Method used to set the focus for a particular instance of a ROI.  
	* @param state A boolean primitive used to determine if the current instance of the ROI 
	* should get the focus.
	*****************************************************************************************/   
	public void setFocus(boolean state);
	public void setSelected(boolean state);

	/*****************************************************************************************
	* Method used to determine if an ROI has the focus.
	* @return True if this ROI has the focus.
	*****************************************************************************************/   
	public boolean hasFocus();

	/*****************************************************************************************
	* Method used to set the status of the ROI.  Status is mainly used in the creation of ROIs.
	* @param s An integer that specifies the state of the ROI. 1=Moving, 2=Resizing, 3=Normal.
	* @see DIDCreationHandler
	*****************************************************************************************/   
	public void setStatus(int s);

	/*****************************************************************************************
	* Method used to get the Status of the ROI.
	* @return An integer that specifies the state of the ROI. 1=Moving, 2=Resizing, 3=Normal.
	*****************************************************************************************/   
	public int getStatus();

	/*****************************************************************************************
	* Method used to set the color of the ROI when it has the focus.
	* @param c A color object to which the ROI who has the focus should be set.
	*****************************************************************************************/   
	public void setFocusColor(Color c);

	/*****************************************************************************************
	* Method used to get the color of the ROI when it has the focus.
	* @return A color object representing the color of the ROI with the focus.
	*****************************************************************************************/   
	public Color getFocusColor();	

	/*****************************************************************************************
	* Method used to set the color of the ROI when it doesn't have the focus.
	* @param c A color object to which the ROI who doesn't have the focus should be set.
	*****************************************************************************************/   
	public void setNoFocusColor(Color c);

	/*****************************************************************************************
	* Method used to get the color of the ROI when it doesn't have the focus.
	* @return A color object representing the color of the ROI with out the focus.
	*****************************************************************************************/   
	public Color getNoFocusColor();	

	/*****************************************************************************************
	* Method used to set the color of the ROI's inside.
	* @param A color object to which the color of the ROI's inside should be set.
	*****************************************************************************************/
	public void setFillColor(Color c);

	/*****************************************************************************************
	* Method used to get the color of the ROI's inside.
	* @return A color object representing the color of the ROI's inside.
	*****************************************************************************************/
	public Color getFillColor();

	/*****************************************************************************************
	* Method used to get the type (Rectangle, Polygon) of a particular implementation of this 
	* class.
	* @return An integer repesenting the type of ROI (Rectangle, Polygon).
	******************************************************************************************/   
	public int getType();

	/*****************************************************************************************
	* Method used to rotate an ROI. 
	* @param myAmount An integer repesenting how far an ROI should be rotated.  Positive 
	* integers rotate the ROI to the left and negative integers rotate the ROI to the right.
	*****************************************************************************************/   
	public void rotate(double myAmount);
	
	/*****************************************************************************************
	* Method used to finalize and ROI.  Finalizing means that any pending transforms relating
	* to this ROI (stored in the 'at' attribute) are calculated and a PathIterator is used to 
	* enumertate the shape and set the ROI's internal coordinate structure to the appropriate
	* values (based on the transform).
	******************************************************************************************/
	public void finalizeROI();

	/*****************************************************************************************
	* Method used to move an active vertex to a new location.
	* @param x Integer representing the new x coordinate for the active vertex.
	* @param y Integer representing the new y coordinate for the active vertex.
	******************************************************************************************/
	public void setActiveVertexCoord(int x, int y);

	/*****************************************************************************************
	* Method used to set the active vertex.  This method should determine if the coordinates
	* passed into it lay within one of the ROIs verticies.  If so, make that vertex the active
	* one and return true, else return false.
	* @param x X coordinate to be iterrigated to see if it lies within the active vertex bounds. 
	* @param y Y coordinate to be iterrigated to see if it lies within the active vertex bounds.
	* @return True if the x,y parameters are within a vertex handle, false if they are not.
	******************************************************************************************/
	public boolean setActiveVertex(int x, int y);

	/*****************************************************************************************
	* Convenience method used move the entire ROI to a new location.  Should be constrained to
	* the active vertex.
	* @param x Integer representing the x coordinate to which the ROI should be moved.
	* @param y Integer representing the y coordinate to which the ROI should be moved.
	******************************************************************************************/
	public void setTranslation(int x, int y);

	/*****************************************************************************************
	* Method used to set a reference to the corresponding DIDNode which the ROI is 
	* representing.
	* @param newAnchorNode A reference to the corresponding DIDNode.
	******************************************************************************************/
	public void setAnchorNode(DIDNode newAnchorNode);

	/*****************************************************************************************
	* Method used to set a reference to the corresponding DIDNode which the ROI is 
	* representing.
	* @param newAnchorNode A reference to the corresponding DIDNode.
	******************************************************************************************/
	public void setAnchorNode(DIDNode newAnchorNode, String color);

	/*****************************************************************************************
	* Method used to return a reference to the corresponding DIDNode.
	* @return DIDNode referenced in attribute list and which this ROI is visually representing.
	******************************************************************************************/
	public DIDNode getAnchorNode();

	/***********************************************************************
	* Method used to set a value of the next ROI.
	***********************************************************************/
	public void setNextRoi(DIDBaseRoi nextRoi);

	/***********************************************************************
	* Method used to return a reference to the next ROI.
	* @return DIDBaseRoi which is the logically next ROI.
	***********************************************************************/
	public DIDBaseRoi getNextRoi();

	/***********************************************************************
	* Method used to set a value of the previous ROI.
	***********************************************************************/
	public void setPrevRoi(DIDBaseRoi prevRoi);

	/***********************************************************************
	/***********************************************************************
	* Method used to return a reference to the previous ROI.
	* @return DIDBaseRoi which is the logically previous ROI.
	***********************************************************************/
	public DIDBaseRoi getPrevRoi();

	/*****************************************************************************************
	* Method used to return the type of data this ROI is reprsenting (Zone, Line, Character).
	* @return An integer (should be a DIDRoiIFC constant) denoting the type of data this
	* ROI represents.
	******************************************************************************************/
	public int getNodeType();

	/*****************************************************************************************
	* Method used to return a string representation of this class.
	* @return A string representation of this object.
	******************************************************************************************/
	public String toString();


	/*****************************************************************************************
	* Method used to synchronize coordinate data between the ROI class and the TreeModel (XML).
	******************************************************************************************/
	public void syncData();


	/*****************************************************************************************
	* Method used to return the number of verticies contained within an ROI.
	* @return An integer representing the number of Verticies contianed withing this ROI.
	******************************************************************************************/
	public int getVertexCount();

	/*****************************************************************************************
	* Method used to determine whether coordinate information is to be displayed next to each 
	* vertex in the ROI.
	* @param dispCoords True if coordinate info is to be displayed next to each vertex, false
	* if no information is to be displayed.
	******************************************************************************************/
	public void setCoordDisp(boolean dispCoords);

	/***********************************************************************
	* Method used to determine whether an arrow is to be displayed from 
	* the current ROI to the logically followed ROI.
	* @param myValue True if an arrow is to be displayed.
	* if false, no information is to be displayed.
	***********************************************************************/
	public void setDrawArrow(boolean myValue);

}
