package did.gui;

import java.awt.Shape;
import java.awt.Point;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.KeyListener;

import java.util.Stack;
import java.util.Vector;
import java.util.Enumeration;

import javax.swing.tree.TreePath;
import javax.swing.event.MouseInputListener;

import did.xml.DIDNode;
import did.xml.DIDDefaultTreeModel;
import did.gui.DIDRoiIFC;
import did.gui.DIDImageCanvas;

/******************************************************************************************
*<pre>
*File:         DIDSelectionHandler.java
*Version:      
*Description:  This class provides the mult-selection capability for the ImageCanvas.  It
*is registered with the ImageCanvas and intercepts mouse events.  When two or more ROIs 
*overlap this class populates the activeRoi attribute in the ImageCanvas object it is 
*registered to with the ROI that should be active.  It accomplishes this by building a
*stack of ROIs that contain the point where the user clicks and then rotating through that
*stack.  
*Note:  This class also extends the KeyListener interface but does not implement any special
*code.  Due to an unresolved issue which does not allow KeyEvent detection on any other 
*object than the main Frame all key detection is being handled in the main Frame.   This 
*is only a stopgap measure.  Once the soure of the error has been identified all KeyEvent
*detection should be moved to the appropriate classes.
*
*Modification Log:
*Date          Name    Comment
*xx-xx-xxxx    XXX	   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
*</pre>
******************************************************************************************/

public class DIDSelectionHandler implements MouseInputListener, KeyListener {

	/** Attribute used to capture the orignal mouse x, y coordinates(to esatablish a point of origin).*/
	private int				xOrigin		= 0, yOrigin		= 0;
	/** Attribute used as temporary storage when determining if a mouseEvent is inside an ROI. */
	private DIDRoiIFC		tmpRoi		= null;
	/** Boolean primitive used as processing flag.  Primitive xyInROI is used to determine if
	mouse x, y 	falls inside a group of ROIs.*/
	private boolean			xyInROI		= false;
	/** Stack data structure used to manage ROI selection.  Object roiStack holds all of the
	ROIs who are contained in the mouse x,y. */
	private Stack			roiStack	= new Stack();
	/** The ROI used to set the next logical entity */
	private DIDRoiIFC nextRoi = null;
	/**  A reference to the main image component. */
	private DIDImageCanvas myCanvas = null;

	/*****************************************************************************************
	* Method used to instantiate the class.  Sets a refernce to the ImageCanvas to which this
	* listener is registered. 
	* @param canvas The ImageCanvas to which this listener is registered. 
	*****************************************************************************************/   
	public DIDSelectionHandler(DIDImageCanvas canvas) {
		myCanvas = canvas;
	}

	/*****************************************************************************************
	* <pre>
	*Captures mouse button press.  This method is the begining of ROI manipulation.  All ROI
	*selection behavior is manipulated through a stack and a vector.  The stack determines 
	*which ROI(s) fall within the current mouse x,y coordinate and the vector contains all 
	*the ROIs of the image.  First, the method checks the existing selection stack to see if
	*it has any members.  If it does then check to see if the current mouse xy is in all the
	*members.  If it is then it is safe to make the next ROI in the stack the current ROI and
	*continue with the mouseDragged event.  If the stack fails any of these tests then a new 
	*one must be built.  Next, if the selection stack is empty OR all of the ROIs in the 
	*selection stack don't contain the latest mouse x,y then it is necessary to (re)build the
	*selection stack.  This is accomplished by looping through the vector containing all the 
	*images ROI's and asking each if the mouse x,y is inside their boundries.  Those that do 
	*are added to the selection stack.  The stack is then interrigated to see if it contains
	*any ROIs.  If it does then current ROI becomes the top ROI in the stack.  If the stack 
	*is empty that means the user has clicked on an empty portion of the image and a new ROI 
	*is created and initalized.
	* </pre>
	* @param e MouseEvent refelecting the current mouse properties.
	* @see #mouseReleased
	*****************************************************************************************/
	public void mousePressed(MouseEvent e) {

		if (myCanvas.activeRoi != null && myCanvas.isSetNext)  {
			if ((e.getModifiers()&e.BUTTON3_MASK) != 0) {
				nextRoi = selectRoi(e);
				myCanvas.activeRoi.setFocus(true);
			} else if ((e.getModifiers()&e.BUTTON1_MASK) != 0) {
				if (nextRoi == null) nextRoi = selectRoi(e);
				myCanvas.activeRoi.setFocus(true);
				if (nextRoi != null) {
					myCanvas.setNext(nextRoi);
					myCanvas.resetFocus();
					myCanvas.activeRoi.setFocus(true);
					nextRoi = null;
				}
				myCanvas.isSetNext = false;
			}
		} else if (myCanvas.myMode != DIDImageCanvas.CREATE &&
				myCanvas.myView != DIDRoiIFC.PAGE) {
			if (!(e.isControlDown() || e.isAltDown() ||
					e.isShiftDown())) {
				
				myCanvas.resetFocus();
				myCanvas.activeRoi = tmpRoi = selectRoi(e);
				if (tmpRoi != null) {
					DIDNode focusNode = tmpRoi.getAnchorNode();
					myCanvas.myTree.setSelectionPath(new TreePath(focusNode.getPath()));
					myCanvas.showInfopanel();
				}
				myCanvas.repaint();
			}
		}
	}

	/***********************************************************************
	* @param
	* @return
	***********************************************************************/
	public DIDRoiIFC selectRoi(MouseEvent e) {
		DIDRoiIFC selectedRoi = null;
                int scaledX = (int)(e.getX()/myCanvas.getScale());
                int scaledY = (int)(e.getY()/myCanvas.getScale());

		selectedRoi = null;
		/** Initialize local variables*/
		xyInROI = false;
		xOrigin = (int)(scaledX);
		yOrigin = (int)(scaledY);

		/** Initialize ROIs */
		myCanvas.resetFocus();
		/* Determine if mouse x,y in existing selection stack.  First
			check to see if stack has any members. */
		if (roiStack.size() > 0) {
			/* Iterate all members of roiStack checking to see
				mouse x,y is them.  If not break and create new
				selection stack.  If yes then set flag.*/
			for (Enumeration enum0 = roiStack.elements() ; enum0.hasMoreElements() ;) {
				tmpRoi = (DIDRoiIFC)enum0.nextElement();
				tmpRoi.finalizeROI();
				if (tmpRoi.contains(xOrigin, yOrigin)) { 
					xyInROI = true;
				} else {
					xyInROI = false;
					break;
				}
			}
			if (xyInROI) {
				tmpRoi = (DIDRoiIFC)roiStack.peek();
				tmpRoi.setFocus(true);
				selectedRoi = tmpRoi;
			}
		} 
		/* If there are no elements in the selection stack OR mouse x,y not in all selecion
		members create new selection stack.*/
		if (roiStack.size() == 0 || xyInROI == false) {
			/* Initialize stack.*/
			roiStack.clear();
			/* Determine which ROI(s) contain mouse xy by
				interating though all ROIs on screen. */ 
			for (Enumeration enum1 = myCanvas.myRoiChildren.elements() ; enum1.hasMoreElements() ;) { 
				tmpRoi = (DIDRoiIFC)enum1.nextElement();
				tmpRoi.finalizeROI();
				if (tmpRoi.contains(xOrigin, yOrigin)) { 
					/* Add ROI who fall in mouse x,y to selection stack */
					roiStack.push(tmpRoi);
				}
			}
			/* Now you have a list of ROI(s) who fall in mouse x,y.  If selection stack empty then
			mouse in untouched area create new ROI.  If stack has members then make the top one the
			active member and determine its state.*/
			if (roiStack.size() > 0) {
				tmpRoi = (DIDRoiIFC)roiStack.peek();
				tmpRoi.setFocus(true);
				selectedRoi = tmpRoi;
			}
		}

		return selectedRoi;
	} //End of class selectRoi


	/*****************************************************************************************
	* Method used to re-initialize the selection model.  This resetting is accomplished by 
	* clearing the roiStack, reseting the focus of all ROIs and setting the activeRoi to null.
	******************************************************************************************/
	public void resetSelection () {
		roiStack.clear();
		myCanvas.resetFocus();
		myCanvas.activeRoi = null;
	}

	/*****************************************************************************************
	* This method is called when the user releases the mouse button.  In this implementation
	* it is necessary to remove the ROI in the top of the Stack.  By doing this the user is 
	* able to cycle through all the RIOs in any given point.
    * @param e MouseEvent refelecting the current mouse properties.
	******************************************************************************************/
	public void mouseReleased(MouseEvent e) {
		if (myCanvas.myMode != DIDImageCanvas.CREATE && myCanvas.myView != DIDRoiIFC.PAGE) {
			if (!(e.isControlDown() || e.isAltDown() || e.isShiftDown())) {
        		if (roiStack.size() > 0) { roiStack.pop(); }
			}
		}
	}

	/*****************************************************************************************
	* This method is called when the mouse is pressed then released.  There is no special 
	* implmentation for this application.
	* @param e MouseEvent refelecting the current mouse properties.
	*****************************************************************************************/ 
	public void mouseClicked(MouseEvent e) { }

	/*****************************************************************************************
	* This method is called when the mouse is being dragged (Mouse pressed then moved while 
	* button is held down within the component.    There is no special implmentation for this
	* application.
	* @param e MouseEvent refelecting the current mouse properties.
	*****************************************************************************************/
	public void mouseDragged(MouseEvent e) {
                int scaledX = (int)(e.getX()/myCanvas.getScale());
                int scaledY = (int)(e.getY()/myCanvas.getScale());

		myCanvas.setMouseCoordinate(scaledX, scaledY);
	}

	/*****************************************************************************************
	* This method is called when the mouse enters the component.  There is no special 
	* implmentation for this application.
	* @param e MouseEvent refelecting the current mouse properties.
	*****************************************************************************************/
	public void mouseEntered(MouseEvent e) { }

	/*****************************************************************************************
	* This method is called when the mouse leaves the component.  There is no special 
	* implmentation for this application.
	* @param e MouseEvent refelecting the current mouse properties.
	*****************************************************************************************/
	public void mouseExited(MouseEvent e) {	}

	/*****************************************************************************************
	* This method is called when the mouse is moving within the component.  There is no 
	* special implmentation for this application.
	* @param e MouseEvent refelecting the current mouse properties.
	*****************************************************************************************/
	public void mouseMoved(MouseEvent e) {
                int scaledX = (int)(e.getX()/myCanvas.getScale());
                int scaledY = (int)(e.getY()/myCanvas.getScale());

		myCanvas.setMouseCoordinate(scaledX, scaledY);
 	}

	/*****************************************************************************************
	* This method is triggered when the user presses a key.  There is no special implmentation
	* for this application.  (See note at top of class)
	* @param e KeyEvent refelecting the current keyboard properties.
	*****************************************************************************************/
	public void keyPressed(KeyEvent e) { } 

	/*****************************************************************************************
	* This method is triggered when the user releases a key.  There is no special implmentation
	* for this application.  (See note at top of class)
	* @param e KeyEvent refelecting the current keyboard properties.
	*****************************************************************************************/
	public void keyReleased(KeyEvent e) { }

	/*****************************************************************************************
	* This method is triggered when the user presses then releases a key.  There is no special
	* implmentation for this application.  (See note at top of class)
	* @param e KeyEvent refelecting the current keyboard properties.
	*****************************************************************************************/
	public void keyTyped(KeyEvent e) { }
}
