package did.gui;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Properties;
import java.util.Enumeration;

import java.awt.Toolkit;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.awt.event.ActionEvent;
import java.awt.event.WindowListener;
import java.awt.event.ActionListener;

import javax.swing.JMenu;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.ButtonGroup;
import javax.swing.JToolBar;
import javax.swing.JTextArea;
import javax.swing.UIManager;
import javax.swing.JSplitPane;
import javax.swing.JScrollPane;
import javax.swing.JOptionPane;
import javax.swing.JTabbedPane;
import javax.swing.JFileChooser;
import javax.swing.SwingUtilities;
import javax.swing.tree.DefaultMutableTreeNode;

import did.io.DIDFileFilter;
import did.gui.DIDControlBar;
import did.xml.DIDTree;
import did.xml.DIDNode;
import did.xml.DIDDefaultTreeModel;

/*******************************************************************************
*<pre>
*File:         TrueViz.java
*Version:      1.1
*Description:  This is the main class from which the entire application is
* launched. This class constructs the visual components of the application.
* This is class also implements three interfaces: WindowListener, ActionListener
* and KeyListener.
*Note: This class also implements the KeyListener interface.  For some unknown
* reason only the main window (this class) is catching keystroke events.
* Certain keystrokes are necessary for the manipulation of the ROIs so this is
* basically a temporary fix until it is discovered why subcomponents are not
* responding to keystrokes.
*
*Modification Log:
*Date          Name    Comment
*xx-xx-xxxx    XXX	   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
*</pre>
*******************************************************************************/

public class TrueViz extends JFrame implements WindowListener, ActionListener, KeyListener {

	/** String Constant used to hold application information. */
	private static final String TITLE = "TruEViz", VERSION = "v1.0";

	/**********************************************************************/
	/** String Constant used to hold the titles for all file menu options.*/
	private static final String FILE = "File", FILEOPEN = "Open",
		FILESAVE = "Save", FILEEXIT = "Exit";

	/** String Constant used to hold the titles for all view menu options.*/
	private static final String VIEW = "View", VIEWIMAGEONLY = "Image Only",
		VIEWLOGICAL = "Logical Relations",
		VIEWFILL = "Fill Bounding Box", VIEWPAGE = "All Entities";
	/** String Constant used to hold the titles for all image menu options. */
	private static final String IMAGE = "Image", IMAGENEW = "New",
		IMAGERECT = "Retangle", IMAGEPOLY = "Polygon",
		IMAGEDEL = "Delete ROI", IMAGESETNEXT = "Set Next Entity",
		IMAGEDELNEXT = "Delete Logical Relation";
	/** String Constant used to hold the titles for all search menu options.
 */
        private static final String SEARCH = "Search", SEARCHFIND = "Find",
		SEARCHFINDDIR = "Find Directory";

	/** A JMenuBar object that holds all the menu options for the application. */
	private JMenuBar	myMenuBar = new JMenuBar();

	/** A JMenu object used to display all the menus used by this application. */
	private JMenu mnuFile, mnuView, mnuImage, mnuImageNew, mnuSearch;

	/** A JMenuItem object used to display all items available under
		the File menu. */
	private JMenuItem	mniFileOpen,	mniFileSave,	mniFileExit;

	/** A JMenuItem object used to display all items available under the View menu. */
	private JCheckBoxMenuItem mniViewLogical, mniViewFill;
	private JRadioButtonMenuItem mniViewImageOnly, mniViewPage;
	private JRadioButtonMenuItem mniViewEntities[];

	/** A JMenuItem object used to display all items available under the Image menu. */
	private JMenuItem mniImageRect, mniImagePoly, mniImageDel;
	private JMenuItem mniImageSetNext, mniImageDelNext;

	/** A JMenuItem object used to display all items available under the Search menu. */
        private JMenuItem mniSearchFind, mniSearchFindDir;

	/**********************************************************************/

	/**********************************************************************/
	/** A Swing SplitPane that holds the main image and data components. */
	private JSplitPane		mainPane		= null;

	/**		*/
	private JPanel		leftPanel		= null;
	public DIDControlBar	controlBar		= null;

	/** A Swing ScrollPane that holds the DIDImageCanvas component responsible for the graphical portion of this app.*/
	private JScrollPane		imageScrollPane = null;
	/** An implementation of JComponent that is responsible for displaying and manipulating ground truth data visually.
	This is accomplished through a heirarchal series of objects called ROIs (Regions of Interest).*/
	private DIDImageCanvas	myCanvas		= null;
	/** A ScrollPane that holds the DIDTree. */
	private JScrollPane		xmlScrollPane	= null;
	/** An Implementation of the Swing DefaultTreeModel class.  This Implementation has specific hooks for handling XML.  
	Specifically, it implements portions of the XML DOM Interface as it relates to this application.*/
	private DIDDefaultTreeModel	myModel		= null;
	/** An implementation of the JTree component responsible for displaying ground truth data (XML file). */
	private DIDTree			myTree			= null;
	/*************************************************************************************************************************/

	/*************************************************************************************************************************/
	/** A Swing JOptionPane object used in conjuction with a Swing JTextArea to display system messages to the user. */
	private JOptionPane		myOptionPane	= new JOptionPane();
	/** A Swing JTextArea object used in conjuction with a Swing JOptionPane to display system messages to the user. */
	private JTextArea		myMessageArea	= null;
	/** A String object used to display an error message.  This particular String is used to inform users that they
	have selected a data file that does not have a corresponding image file.*/
	private static final String	errLoadImage = "Unable to locate corresponding image file.  Please make sure your data file " + 
												"and image file are in the same directory.  Note:  This version only supports " +
												"Group 3 & 4 compressed TIFFs.";
	/** A File object used to store reference to the data file currently being edited.   */
	private File myDataFile		= null;
	/*************************************************************************************************************************/

	/******************************************************************************************
	* Constructor used to initialize the application.  Sets up listeners, builds components 
	* and initializes main window.
	* @see #buildComponents
	* @see #buildMenuBar
	******************************************************************************************/
	public TrueViz(String filename) {
		super(TITLE + " " + VERSION);
		if (filename != null) {
			myDataFile = new File(filename);
		}
		/** Register this class to recieve window events (opened, closed, activated or deactivated,
		iconified or deiconified). */
		addWindowListener(this);
		/** Register this class to recieve action events gerenated by components register to this class. */
		addKeyListener(this);
		setJMenuBar(buildMenuBar());
		buildComponents();
		setLocation(0,0);
		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		setSize((int)(dim.getWidth()*0.85), (int)(dim.getHeight()*0.85));
		show();
		if (filename != null) {
			fileOpen(myDataFile);
		}
	}

	/******************************************************************************************
	* This method builds the menu bar for the application as well as registering the appropriate
	* menu items as ActionListeners to this class.  That way when the menu item generates an 
	* action event (in response to user interaction i.e. selecting the item) the component that 
	* orginated the event can be identified and appropriate code envoked.
	* Based on the item specific action(s) are performed.
	* @return A JMenuBar with the appropriate layout for the application.
	* @see #actionPerformed
	*******************************************************************************************/
	private JMenuBar buildMenuBar() {
		mnuFile		= new JMenu(FILE);
		mniFileOpen = new JMenuItem(FILEOPEN);
		mniFileOpen.addActionListener(this);
		mnuFile.add(mniFileOpen);
		mniFileSave = new JMenuItem(FILESAVE);
		mniFileSave.addActionListener(this);
		mniFileSave.setEnabled(false);
		mnuFile.add(mniFileSave);
		mniFileExit = new JMenuItem(FILEEXIT);
		mniFileExit.addActionListener(this);
		mnuFile.add(mniFileExit);
		myMenuBar.add(mnuFile);

		mnuView = new JMenu(VIEW);
		mnuView.setEnabled(false);
		myMenuBar.add(mnuView);

		mnuImage = new JMenu(IMAGE);
		mnuImage.setEnabled(false);
		mnuImageNew  = new JMenu(IMAGENEW);
		mniImageRect = new JMenuItem(IMAGERECT);
		mniImageRect.addActionListener(this);
		mnuImageNew.add(mniImageRect);
		mniImagePoly = new JMenuItem(IMAGEPOLY);
		mniImagePoly.addActionListener(this);
		mnuImageNew.add(mniImagePoly);
		mnuImage.add(mnuImageNew);
		mniImageDel = new JMenuItem(IMAGEDEL);
		mniImageDel.addActionListener(this);
		mnuImage.add(mniImageDel);
		mnuImage.addSeparator();
		mniImageSetNext = new JMenuItem(IMAGESETNEXT);
		mniImageSetNext.addActionListener(this);
		mnuImage.add(mniImageSetNext);
		mniImageDelNext = new JMenuItem(IMAGEDELNEXT);
		mniImageDelNext.addActionListener(this);
		mnuImage.add(mniImageDelNext);
		myMenuBar.add(mnuImage);

                mnuSearch = new JMenu(SEARCH);
                mniSearchFind = new JMenuItem(SEARCHFIND);
                mniSearchFind.setEnabled(false);
                mniSearchFind.addActionListener(this);
                mnuSearch.add(mniSearchFind);
                mniSearchFindDir = new JMenuItem(SEARCHFINDDIR);
                mniSearchFindDir.addActionListener(this);
                mnuSearch.add(mniSearchFindDir);
                myMenuBar.add(mnuSearch);

		return(myMenuBar);
	}

	/***********************************************************************
	* This method build menu items for View menu which are customized for
	* the XML files.
	***********************************************************************/
	private void buildViewMenu() {
		mnuView.removeAll();
		ButtonGroup viewButtonGroup = new ButtonGroup();

		mniViewImageOnly = new JRadioButtonMenuItem(VIEWIMAGEONLY);
		mniViewImageOnly.addActionListener(this);
		mnuView.add(mniViewImageOnly);
		mniViewPage = new JRadioButtonMenuItem(VIEWPAGE);
		mniViewPage.addActionListener(this);
		mnuView.add(mniViewPage);

		viewButtonGroup.add(mniViewImageOnly);
		viewButtonGroup.add(mniViewPage);
		int len = myModel.EntityType.length();
		mniViewEntities = new JRadioButtonMenuItem[len];
		for (int i=0; i < myModel.EntityType.length(); i++) {
			String name = myModel.EntityType.getName(i);
			mniViewEntities[i] = new JRadioButtonMenuItem(name);
			mniViewEntities[i].addActionListener(this);
			mnuView.add(mniViewEntities[i]);
			viewButtonGroup.add(mniViewEntities[i]);
		}

		mnuView.addSeparator();
		mniViewLogical = new JCheckBoxMenuItem(VIEWLOGICAL);
		mniViewLogical.addActionListener(this);
		mnuView.add(mniViewLogical);
		mniViewFill = new JCheckBoxMenuItem(VIEWFILL);
		mniViewFill.addActionListener(this);
		mnuView.add(mniViewFill);
	}


	/******************************************************************************************
	* This method builds the data and graphical componets for the app and adds them to the 
	* main window.  Note: The construction of the DIDImageCanvas component contains a static 
	* reference to a test file.  This static reference will be eliminated in the production
	* version.
	*******************************************************************************************/
	private void buildComponents() {
		myModel = new DIDDefaultTreeModel(new DefaultMutableTreeNode(),
					new String("Awaiting User Input..."));
		myTree = new DIDTree(myModel);
		myTree.setEditable(false);

		leftPanel = new JPanel();
		xmlScrollPane	= new JScrollPane(myTree);
		myCanvas = new DIDImageCanvas(null, 1, myTree, this);
		controlBar = new DIDControlBar(this, myCanvas);
		imageScrollPane = new JScrollPane(myCanvas);
		leftPanel.setLayout(new BorderLayout());
		leftPanel.add(controlBar, BorderLayout.NORTH);
		leftPanel.add(imageScrollPane, BorderLayout.CENTER);
		mainPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, leftPanel, xmlScrollPane);
		getContentPane().add(mainPane);	
	}

	/**********************************************************************
	* This method scale the image so that the width of the image fit to
	* the width of the panel.
	**********************************************************************/
	public void fitWidth() {
		Rectangle rect = imageScrollPane.getViewportBorderBounds();
		Dimension dim = myCanvas.getPreferredScrollableViewportSize();
		float newscale = (float)rect.getWidth()/(float)dim.getWidth();
		newscale *= myCanvas.getScale();
		myCanvas.changeScale(newscale);
	}

	/**********************************************************************
	* This method scale the image so that the height of the image fit to
	* the height of the panel.
	**********************************************************************/
	public void fitHeight() {
		Rectangle rect = imageScrollPane.getViewportBorderBounds();
		Dimension dim = myCanvas.getPreferredScrollableViewportSize();
		float newscale = (float)rect.getHeight()/(float)dim.getHeight();
		newscale *= myCanvas.getScale();
		myCanvas.changeScale(newscale);
	}

	/**********************************************************************
	* This method opens a file.
	* @param selectedFile The file to be opened
	**********************************************************************/
	public void fileOpen(File selectedFile) {
		File tmpFile = selectedFile;
		boolean openSuccessful = true;

		if (tmpFile.getName().toUpperCase().endsWith(".XML")) {
			StringBuffer tmpImageBuffer = new StringBuffer(tmpFile.getPath());
			tmpImageBuffer.replace(tmpImageBuffer.length() - 3, tmpImageBuffer.length(), "tif");
			File tmpImageFile = new File(tmpImageBuffer.toString());
			if (tmpImageFile.exists()) {
				myCanvas.reset();
				myCanvas.setImage(tmpImageFile.getAbsolutePath());
				myModel.buildDIDModel(selectedFile.getAbsolutePath());
				myCanvas.resetFocus();
				myDataFile = tmpFile;
			} else {
				myMessageArea = new JTextArea(errLoadImage);
				myMessageArea.setColumns(45);
				myMessageArea.setLineWrap(true);
				myMessageArea.setEditable(false);
				myOptionPane.showMessageDialog(this, myMessageArea);
				openSuccessful = false;
			}
		} else if (tmpFile.getName().toUpperCase().endsWith(".TIF")) {
			StringBuffer tmpDataBuffer = new StringBuffer(tmpFile.getPath());
			tmpDataBuffer.replace(tmpDataBuffer.length() - 3, tmpDataBuffer.length(), "xml");
			File tmpDataFile = new File(tmpDataBuffer.toString());
			myCanvas.reset();
			myCanvas.setImage(selectedFile.getAbsolutePath());
			if (tmpDataFile.exists()) {
				myModel.buildDIDModel(tmpDataFile.getAbsolutePath());
				myDataFile = tmpDataFile;
			} else {
				myModel.buildDIDModel(DIDNode.PAGE + ".xml");
				myDataFile = new File(tmpDataBuffer.toString());
			}
			myCanvas.resetFocus();
		} 
		/** If the open was sucessful then initialize the app to
			a state ready for user input. */
		if (openSuccessful) {
			fitWidth();
			myCanvas.setMode(DIDImageCanvas.EDIT);
			myCanvas.setView(DIDRoiIFC.ZONE);
			myTree.setEditable(true);
			mniFileSave.setEnabled(true);
			buildViewMenu();
			mnuView.setEnabled(true);
			mnuImage.setEnabled(false);
			mniSearchFind.setEnabled(true);
			mnuImageNew.setText(IMAGENEW);
			mniViewEntities[0].setSelected(true);
			mnuImage.setEnabled(true);
			mniViewLogical.setState(false);
			mniViewFill.setState(false);
		}
		myCanvas.repaint();
	}

	/******************************************************************************************
	* This method intercepts actions from resitered components and performs specific tasks 
	* based on the component the action is coming from.  Some of the main tasks include image 
	* manipulation, data manipulation, and file maintenance.
	* @param e A semantic event which indicates that a component-defined action occured.
	******************************************************************************************/
	public void actionPerformed(ActionEvent e) {
		/** This portion creates a filechooser for the user to select either an image or data file
		to be displayed.  If a data file is selected then the app tries to load the corresponding image
		file. Correspondence is determined by the two files having the same file name with different 
		extensions (.xml & .tif).  If the image file does not exist then and error message is displayed. 
		If an image file is selected then the app tries to load the corresponding data file.  If one does
		not exist then one is created for the user. */
		if (e.getActionCommand() == FILEOPEN) {
			JFileChooser myChooser = new JFileChooser();
			DIDFileFilter filter = new DIDFileFilter(new String [] {"xml", "tif"}, "XML & TIF File Types");
			myChooser.addChoosableFileFilter(filter);
			myChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
			if (myDataFile != null) { myChooser.setCurrentDirectory(myDataFile.getParentFile()); }
			switch (myChooser.showOpenDialog(this)) {
				case JFileChooser.APPROVE_OPTION :
					File tmpFile = myChooser.getSelectedFile();
					fileOpen(tmpFile);
				break;
				case JFileChooser.CANCEL_OPTION :
				break;
			}
		} else if (e.getActionCommand() == FILEEXIT) {
			/** Exit the application. */
			closeApplication();
		} else if (e.getActionCommand() == FILESAVE) {
			/** This portion of code is responsible for saving the data object(XML) back to the file system. */
			JFileChooser myChooser = new JFileChooser();
			DIDFileFilter filter = new DIDFileFilter(new String [] {"xml"}, "XML File Types");
			myChooser.addChoosableFileFilter(filter);
			myChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
			myChooser.setSelectedFile(myDataFile);
			switch (myChooser.showSaveDialog(this)) {
				case JFileChooser.APPROVE_OPTION :
					myModel.saveModel(myChooser.getSelectedFile().getAbsolutePath());
				break;
				case JFileChooser.CANCEL_OPTION :
				break;
			}
		} else if (e.getActionCommand() == VIEWIMAGEONLY) {
				myCanvas.setMode(DIDImageCanvas.IMAGE_ONLY);
			myCanvas.repaint();
		} else if (e.getActionCommand() == VIEWPAGE) {
			/** Set the ImageCanvas view mode to the Page level. */
			myCanvas.setMode(DIDImageCanvas.EDIT);
			myCanvas.setView(DIDRoiIFC.PAGE);
			mnuImageNew.setText(IMAGENEW);
			mnuImage.setEnabled(false);
			myCanvas.setFillColors(mniViewFill.getState());
			myCanvas.repaint();
		} else if (e.getActionCommand() == VIEWLOGICAL) {
			myCanvas.setArrowDisplay(DIDRoiIFC.PAGE, mniViewLogical.getState());
			myCanvas.repaint();
		} else if (e.getActionCommand() == VIEWFILL) {
			myCanvas.setFillColors(mniViewFill.getState());
			myCanvas.repaint();
		} else if (e.getActionCommand() == IMAGERECT) {
			/** Create a new rectangular ROI. */
			myCanvas.newRoi(DIDRoiIFC.RECTANGLE);
		} else if (e.getActionCommand() == IMAGEPOLY) {
			/** Create a new ROI polygon. */
			myCanvas.newRoi(DIDRoiIFC.POLYGON);
		} else if (e.getActionCommand() == IMAGEDEL) {
			/** Delete the selected ROI polygon. */
			myCanvas.deleteRoi();
		} else if (e.getActionCommand() == IMAGESETNEXT) {
			/** Set the next entity of the selected entity. */
			myCanvas.setNext();
		} else if (e.getActionCommand() == IMAGEDELNEXT) {
			/** Delete the logical relation of the selected entity. */
			myCanvas.deleteNext();
		} else if (e.getActionCommand() == SEARCHFIND) {
			myCanvas.setMode(DIDImageCanvas.EDIT);
			myCanvas.setView(DIDRoiIFC.PAGE);
			mnuImageNew.setText(IMAGENEW);
			mnuImage.setEnabled(false);
			mniViewPage.setSelected(true);
			mniViewFill.setState(false);
			myCanvas.setFillColors(mniViewFill.getState());

			myCanvas.searchFind();
                } else if (e.getActionCommand() == SEARCHFINDDIR) {
			myCanvas.searchFindDir();
		} else {
			String command = e.getActionCommand();
			myCanvas.setMode(DIDImageCanvas.EDIT);
			/** Set the ImageCanvas view mode. */
			int type = myModel.EntityType.nameToType(command);
			if (myModel.EntityType.isValid(command) && 
					myCanvas.setView(type)) {
				mnuImageNew.setText(IMAGENEW + " " + command);
				for (int i=0; i<myModel.EntityType.length(); i++) {
					mniViewEntities[i].setEnabled(true);
				}
				mnuImage.setEnabled(true);
				myCanvas.setFillColors(mniViewFill.getState());
				myCanvas.repaint();
			}
		}
	}

	/******************************************************************************************
	* Method used to terminate the application.  This method displays a message to the System
	* out stream, gets an instance of the DIDPorperties and then saves this instance to the 
	* appropriate directory.
	******************************************************************************************/
	private void closeApplication()	{
		System.out.println("Terminating Program");
		Properties props = DIDProperties.getInstance();
		try {
			FileOutputStream myOutput = new FileOutputStream(new String("trueviz.properties"));
			props.store(myOutput, "TruEViz Properties: v1.0");
			myOutput.close();
		} catch(Exception e){
			System.out.println("Unable to save Properties.  Error: " + e);
		}
		System.exit(0);
	}

	/******************************************************************************************
	* This method is triggered when the window opens and captures all related events.  There is 
	* no special implmentation for this application.
	* @param e WindowEvent refelecting the current windows properties.
	******************************************************************************************/
	public void windowOpened(WindowEvent e)			{ }

	/******************************************************************************************
	* This method is triggered when the window is closed and captures all related events. There 
	* is no special implmentation for this application.
	* @param e WindowEvent refelecting the current windows properties.
	******************************************************************************************/
	public void windowClosed(WindowEvent e)			{ }

	/******************************************************************************************
	* This method is triggered when the window is in the process of being closed and captures
	* all related events.  In this implementation, closing of this window signifies the 
	* termination of the application.
	* @param e WindowEvent refelecting the current windows properties.
	******************************************************************************************/
	public void windowClosing(WindowEvent e)		{ closeApplication(); }

	/*****************************************************************************************
	* This method is triggered when the window activated.  There is no special implmentation 
	* for this application. 
	* @param e WindowEvent refelecting the current windows properties.
	*****************************************************************************************/
	public void windowActivated(WindowEvent e)		{ }

	/******************************************************************************************
	* This method is triggered when the window Deactivated.  There is no special implmentation
	* for this application.
	* @param e WindowEvent refelecting the current windows properties.
	******************************************************************************************/
	public void windowDeactivated(WindowEvent e)	{ }

	/*****************************************************************************************
	* This method is triggered when the window Iconified.  There is no special implmentation
	* for this application.
	* @param e WindowEvent refelecting the current windows properties.
	*****************************************************************************************/
	public void windowIconified(WindowEvent e)		{ }

	/*****************************************************************************************
	* This method is triggered when the window Deiconified.  There is no special implmentation
	* for this application.
	* @param e WindowEvent refelecting the current windows properties.
	*****************************************************************************************/
	public void windowDeiconified(WindowEvent e)	{ }

	/*****************************************************************************************
	* This method is triggered when the user presses a key.  In this implementation the comma
	* key is used to rotate ROIs left and the period key is used to rotate ROIs right.  
	* Note:  These are only stopgap measures until KeyEvents can be properly detected and 
	* delegated by the DIDImageCanvas.
	* @param e KeyEvent refelecting the current keyboard properties.
	*****************************************************************************************/
	public void keyPressed(KeyEvent e) {
		switch (e.getKeyCode()) {
			case KeyEvent.VK_COMMA:
				myCanvas.rotateLeft();
				myCanvas.repaint();
			break;
			case KeyEvent.VK_PERIOD:
				myCanvas.rotateRight();
			break;
			case KeyEvent.VK_DELETE:
				myCanvas.deleteRoi();
			break;
			case KeyEvent.VK_PLUS:
			case KeyEvent.VK_EQUALS:
			case KeyEvent.VK_PAGE_UP:
				myCanvas.zoomIn();
			break;
			case KeyEvent.VK_MINUS:
			case KeyEvent.VK_PAGE_DOWN:
				myCanvas.zoomOut();
			break;
		}
	}

	/*****************************************************************************************
	* This method is triggered when the user releases a key.  There is no special implmentation
	* for this application.
	* @param e KeyEvent refelecting the current keyboard properties.
	*****************************************************************************************/
	public void keyReleased(KeyEvent e)		{ }

	/*****************************************************************************************
	* This method is triggered when the user presses then releases a key.  There is no special
	* implmentation for this application.
	* @param e KeyEvent refelecting the current keyboard properties.
	*****************************************************************************************/
	public void keyTyped(KeyEvent e)		{ }

	/*****************************************************************************************
	* This method is triggered when this class is instantied by the virtual machine.  It is 
	* used to instantiate this object and then manually calls garbage collection to speed up 
	* the disposal of temporary objects used in starting up the application.
	* @param args A String array representing all command line arguments that are to be 
	* passed to this class.
	*****************************************************************************************/
	public static void main(String[] args) {
		if (args.length > 0) {
			new TrueViz(args[0]);
		} else {
			new TrueViz(null);
		}
		System.gc();
	}
}

