package did.io;

import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.FileNotFoundException;
import java.io.UnsupportedEncodingException;

import org.w3c.dom.Attr;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;

import com.ibm.xml.internal.MIME2Java;

/*****************************************************************************************
*<pre>
*File:         DOMWriter.java
*Version:      1.1
*Description:  This class is used to traverse a DOM tree in order to write a document to an
*outputstream.  
*NOTE: The print() method is adding some erroneous line feeds which make the data soemwhat
*unreadable in raw form without the use of an editor.  This method should be fixed in 
*future versions. 
*
*Modification Log:
*Date          Name    Comment
*xx-xx-xxxx    XXX     xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
*</pre>
*****************************************************************************************/

public class DOMWriter {

	/** Default parser name. */
	private static final String DEFAULT_PARSER_NAME = "dom.wrappers.DOMParser";

	/** Default Encoding */
	private static  String PRINTWRITER_ENCODING = "UTF8";

	private static String MIME2JAVA_ENCODINGS[] =
		{ "Default", "UTF-8", "US-ASCII", "ISO-8859-1", "ISO-8859-2", "ISO-8859-3", "ISO-8859-4", 
		"ISO-8859-5", "ISO-8859-6", "ISO-8859-7", "ISO-8859-8", "ISO-8859-9", "ISO-2022-JP",
		"SHIFT_JIS", "EUC-JP","GB2312", "BIG5", "EUC-KR", "ISO-2022-KR", "KOI8-R", "EBCDIC-CP-US", 
		"EBCDIC-CP-CA", "EBCDIC-CP-NL", "EBCDIC-CP-DK", "EBCDIC-CP-NO", "EBCDIC-CP-FI", "EBCDIC-CP-SE",
		"EBCDIC-CP-IT", "EBCDIC-CP-ES", "EBCDIC-CP-GB", "EBCDIC-CP-FR", "EBCDIC-CP-AR1", 
		"EBCDIC-CP-HE", "EBCDIC-CP-CH", "EBCDIC-CP-ROECE","EBCDIC-CP-YU",  
		"EBCDIC-CP-IS", "EBCDIC-CP-AR2", "UTF-16" };

	/** Canonical output. */
	protected boolean canonical;

	/** Print writer. */
	protected PrintWriter out;

	/** File to be written to. */
	protected FileOutputStream myStream = null;

	public DOMWriter(String fileName) throws UnsupportedEncodingException, FileNotFoundException {
		try {
			myStream = new FileOutputStream(fileName);
		} catch(Exception e) {
			System.out.println("Error Writing file: " + e);
		}
		out = new PrintWriter(new OutputStreamWriter(myStream, PRINTWRITER_ENCODING));
		this.canonical = canonical;
	}
   
	public DOMWriter(String encoding, boolean canonical) throws UnsupportedEncodingException {
		out = new PrintWriter(new OutputStreamWriter(System.out, encoding));
		this.canonical = true;
	} // <init>(String,boolean)

	/** Default constructor. */
	public DOMWriter(boolean canonical) throws UnsupportedEncodingException {
		this( getWriterEncoding(), canonical);
	}

	public static String getWriterEncoding( ) {
		return (PRINTWRITER_ENCODING);
	}// getWriterEncoding 

	public static void  setWriterEncoding( String encoding ) {
		if( encoding.equalsIgnoreCase( "DEFAULT" ) )
			PRINTWRITER_ENCODING  = "UTF8";
		else if( encoding.equalsIgnoreCase( "UTF-16" ) )
			PRINTWRITER_ENCODING  = "Unicode";
		else
			PRINTWRITER_ENCODING = MIME2Java.convert( encoding ); 
	}// setWriterEncoding 


	public static boolean isValidJavaEncoding( String encoding ) {
		for ( int i = 0; i < MIME2JAVA_ENCODINGS.length; i++ ) {
			if ( encoding.equals( MIME2JAVA_ENCODINGS[i] ) ) {
				return (true);
			}
		}
		return (false);
	}// isValidJavaEncoding 


	/** Prints the resulting document tree. */
	public static void print(String parserWrapperName, String uri, boolean canonical ) {
		try {
			DOMParserWrapper parser = (DOMParserWrapper)Class.forName(parserWrapperName).newInstance();
			Document document = parser.parse(uri);
			DOMWriter writer = new DOMWriter(canonical);
			writer.print(document);
		} catch ( Exception e ) {
			e.printStackTrace(System.err);
		}
	} // print(String,String,boolean)


	/** Prints the specified node, recursively. */
	public void print(Node node) {
		if ( node == null ) { return; }

		int type = node.getNodeType();
		switch ( type ) {
			// print document
			case Node.DOCUMENT_NODE: {
				if ( !canonical ) { 
					String  Encoding = this.getWriterEncoding();
					if( Encoding.equalsIgnoreCase( "DEFAULT" ) )
						Encoding = "UTF-8";
					else if( Encoding.equalsIgnoreCase( "Unicode" ) )
						Encoding = "UTF-16";
				else 
					Encoding = MIME2Java.reverse( Encoding );
					out.println("<?xml version=\"1.0\" encoding=\"" +  Encoding + "\"?>");
					out.println("<!DOCTYPE Page SYSTEM \"Trueviz.dtd\">");

				}
				print(((Document)node).getDocumentElement());
				out.flush();
				break;
			}
			// print element with attributes
			case Node.ELEMENT_NODE: {
				out.print('<');
				out.print(node.getNodeName());
				Attr attrs[] = sortAttributes(node.getAttributes());
				for ( int i = 0; i < attrs.length; i++ ) {
					Attr attr = attrs[i];
					out.print(' ');
					out.print(attr.getNodeName());
					out.print("=\"");
					out.print(normalize(attr.getNodeValue()));
					out.print('"');
				}
				out.println('>');
				NodeList children = node.getChildNodes();
				if ( children != null ) {
					int len = children.getLength();
					for ( int i = 0; i < len; i++ ) {
						print(children.item(i));
					}
				}
				break;
			}

            // handle entity reference nodes
			case Node.ENTITY_REFERENCE_NODE: {
				if ( canonical ) {
					NodeList children = node.getChildNodes();
					if ( children != null ) {
						int len = children.getLength();
						for ( int i = 0; i < len; i++ ) {
							print(children.item(i));
						}
					}
				} else {
					out.print('&');
					out.print(node.getNodeName());
					out.print(';');
				}
				break;
			}

            // print cdata sections
			case Node.CDATA_SECTION_NODE: {
				if ( canonical ) {
					out.print(normalize(node.getNodeValue()));
				} else {
					out.print("<![CDATA[");
					out.print(node.getNodeValue());
					out.println("]]>");
				}
				break;
			}

			// print text
			case Node.TEXT_NODE: {
				out.print(normalize(node.getNodeValue()));
				break;
			}

			// print processing instruction
			case Node.PROCESSING_INSTRUCTION_NODE: {
				out.print("<?");
				out.print(node.getNodeName());
				String data = node.getNodeValue();
				if ( data != null && data.length() > 0 ) {
					out.print(' ');
					out.print(data);
				}
				out.println("?>");
				break;
			}
		}
		if ( type == Node.ELEMENT_NODE ) {
			out.print("</");
			out.print(node.getNodeName());
			out.println('>');
		}
		out.flush();
	} // print(Node)

	/** Returns a sorted list of attributes. */
	protected Attr[] sortAttributes(NamedNodeMap attrs) {
		int len = (attrs != null) ? attrs.getLength() : 0;
		Attr array[] = new Attr[len];
		for ( int i = 0; i < len; i++ ) {
			array[i] = (Attr)attrs.item(i);
		}
		for ( int i = 0; i < len - 1; i++ ) {
			String name  = array[i].getNodeName();
			int index = i;
			for ( int j = i + 1; j < len; j++ ) {
				String curName = array[j].getNodeName();
				if ( curName.compareTo(name) < 0 ) {
					name  = curName;
					index = j;
				}
			}
			if ( index != i ) {
				Attr temp    = array[i];
				array[i]     = array[index];
				array[index] = temp;
			}
		}
		return (array);
	} // sortAttributes(NamedNodeMap):Attr[]

	/** Normalizes the given string. */
	protected String normalize(String s) {
		StringBuffer str = new StringBuffer();
		int len = (s != null) ? s.length() : 0;
		for ( int i = 0; i < len; i++ ) {
			char ch = s.charAt(i);
			switch ( ch ) {
				case '<': {
					str.append("&lt;");
					break;
				}
				case '>': {
					str.append("&gt;");
					break;
				}
				case '&': {
					str.append("&amp;");
					break;
				}
				case '"': {
					str.append("&quot;");
					break;
				}
				case '\r':
				case '\n': {
					if ( canonical ) {
						str.append("&#");
						str.append(Integer.toString(ch));
						str.append(';');
						break;
					}// else, default append char
				}
				default: {
					str.append(ch);
				}
			}
		}
		return (str.toString());
	} // normalize(String):String

	/** Prints the usage. */
	private static void printUsage() {
		System.err.println("usage: java dom.DOMWriter (options) uri ...");
		System.err.println();
		System.err.println("options:");
		System.err.println("  -p name  Specify DOM parser wrapper by name.");
		System.err.println("           Default parser: " + DEFAULT_PARSER_NAME);
		System.err.println("  -c       Canonical XML output.");
		System.err.println("  -h       This help screen.");
		System.err.println("  -e       Output Java Encoding.");
		System.err.println("           Default encoding: UTF-8");
	} // printUsage()

	private static void printValidJavaEncoding() {
		System.err.println( "    ENCODINGS:" );
		System.err.print( "   " );
		for( int i = 0; i < MIME2JAVA_ENCODINGS.length; i++) {
			System.err.print( MIME2JAVA_ENCODINGS[i] + " " );
			if( (i % 7 ) == 0 ){
				System.err.println();
				System.err.print( "   " );
			}
		}
	} // printJavaEncoding()            
}
