package did.xml;

import java.awt.Component;
import java.awt.event.MouseEvent;

import java.util.EventObject;

import javax.swing.JTree;
import javax.swing.tree.TreePath;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.DefaultTreeCellEditor;
import javax.swing.tree.DefaultTreeCellRenderer;

import did.xml.DIDNode;

/*****************************************************************************************
*<pre>
*File:         DIDTreeCellEditor.java
*Version:      1.1
*Description:  This class is called when the user triple clicks on a node of the DIDTree.
*An editing facility (the default is a text box) through with the value of the node can be
*changed is then passed back to the DIDTree which renders it using the correct icon from
*the TreeCellRenderer.
*NOTE: Used in conjuction with the valueForPathChanged() method of the TreeModel, this class
*can alter the value of a DOM node.  Since the toString() method of the DIDNode returns the 
*DOM node's name and value, changing the DOM node's value also has the effect of simultaneously
*changing the display properties of the node to which it belongs.
*
*Modification Log:
*Date          Name    Comment
*xx-xx-xxxx    XXX	   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
*</pre>
******************************************************************************************/

public class DIDTreeCellEditor extends DefaultTreeCellEditor {

	/*****************************************************************************************
	* Method used to instantiate this class. 
	* @param tree A reference to the JTree for which this class will render editing components. 
	*****************************************************************************************/   
	public DIDTreeCellEditor(JTree tree) {
		super(tree, (DefaultTreeCellRenderer)(tree.getCellRenderer()));
	}

	/*****************************************************************************************
	* Method used to configure the editor which is passed onto the realEditor.  In this 
	* implementation the value placed in the editor is set by retrieving the value of the 
	* DIDNode's DOM node.  
	* @parm table The JTree that is asking the editor to edit This parameter can be null.
	* @parm value The value of the cell to be edited.
	* @parm isSelected True is the cell is to be renderer with selection highlighting.
	* @parm expanded True if the node is expanded.
	* @parm leaf True if the node is a leaf node.
	* @parm row The row index of the node being edited.
	* @return The component for editing.
	* @see DIDDefaultTreeModel#valueForPathChanged
	*****************************************************************************************/   
	public Component getTreeCellEditorComponent(JTree tree, Object value, boolean isSelected,
												boolean expanded, boolean leaf, int row) {
		String nodeValue = null;
		if (value instanceof DIDNode && value != null) {
			DIDNode myDIDNode = (DIDNode)value;
			nodeValue = myDIDNode.getNodeValue();
		}
		return(super.getTreeCellEditorComponent(tree, nodeValue, isSelected, expanded, leaf, row));
	}

	/*****************************************************************************************
	* Method used to determine if a particular cell should be editable.  In this implementation
	* editability is determine by the position of the node on the tree.  Only leaf nodes that
	* do not have node name of "x" or "y" should be editable.
	* NOTE: Not allowing the user to exit node with names of x or y solves a problem of syncing
	* the XML with the ROI.  This is a temporary fix as a efficient method was not able to be
	* developed in the time allotted.
	* @param e The event the editor should use to consider whether to begin editing or not.
	* @return True if editing can be started.
	*****************************************************************************************/   
	public boolean isCellEditable(EventObject e) {
		if (super.isCellEditable(e) && e instanceof MouseEvent) {
			TreePath path = tree.getPathForLocation(((MouseEvent)e).getX(), ((MouseEvent)e).getY());
			DIDNode tmpNode = (DIDNode)path.getLastPathComponent();
			if (tmpNode.isLeaf() && (tmpNode.getNodeName().equals("x") || tmpNode.getNodeName().equals("y"))) {
				return false;
			} else {
				return tmpNode.isLeaf();
			}
		}
		return false;
	}
}