package did.xml;

import javax.swing.tree.DefaultMutableTreeNode;

import org.w3c.dom.Node;

import did.gui.DIDRoiIFC;
import did.xml.DIDDefaultTreeModel;

/*****************************************************************************************
*<pre>
*File:         DIDNode.java
*Version:      1.1
*Description:  This class extends the DefaultMutableTreeNode class and therefore fits nicely
*into the Swing MVC concept. Used in conjuction with the TreeModel and JTree (and the proper
*listeners) this class provides the basis for displaying and editing the DOM tree and 
*underlying XML data.  For every Node in the DOM (built from the XML data file) there is a 
*corresponding DIDNode.  Since the DOM can not be directly used in the Swing MVC 
*architecture this parralleling of tree structures allows the application to use existing
*swing components (JTree, DefaultTreeModel, various TreeListeners) to view and manipulate
*DOM (XML) specific data.
*
*Modification Log:
*Date          Name    Comment
*xx-xx-xxxx    XXX	   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
*</pre>
******************************************************************************************/

public class DIDNode extends DefaultMutableTreeNode {

	/** Attribute that denotes the type of data this Node contains as it relates to the application 
	(Page, Zone, Line, Character)*/
	private String dataType = null;
	/** Attribute that points to the ROI that represents this Node in the ImageCanvas effectively 
	creating a bi-directional link.*/
	private DIDRoiIFC myRoi = null;
	/** The TreeModel this Node belongs to.  Used when the useObject (DOM Node) changes to fire 
	off the appropriate sequence of events so that the GUI updates properly. */
	private DIDDefaultTreeModel myModel = null;
	/** Contant(s) that denote the type of data this Node represents as it pertains to the application. */
	public static String DOCUMENT = "Document", PAGE = "Page", ZONE = "Zone", WORD = "Word", LINE = "Line",
			CHARACTER = "Character", VERTEX = "Vertex", OTHER = "Other";

	/*****************************************************************************************
	* Method used to instantiate this class.  This method sets the user object to the DOM Node
	* this node is responsible for displaying as well as setting a refernce to the TreeModel 
	* this node is a part of.
	* @param newNode New W3C Node object this node is to display.
	* @param newModel TreeModel which this node is a part of.
	*****************************************************************************************/   
	public DIDNode(Node newNode, DIDDefaultTreeModel newModel) {
		super(newNode);
		setDataType();
		myModel = newModel;
	}

	/*****************************************************************************************
	* Method used to set the userObject attribute to the DOM node this node is mirroring.
	* @param newNode A W3C node to be place in the userObject attribute.
	*****************************************************************************************/   
	public void setDomNode(Node newNode) { 
		setUserObject(newNode);
	}

	/*****************************************************************************************
	* Method used to return the DOM node this node is mirroring.
	* @return A W3C Node object which this class is parralleling .
	*****************************************************************************************/   
	public Node getDomNode() { return (Node)getUserObject(); }

	/*****************************************************************************************
	* Method used to return the name of the DOM node in the userObject.
	* @return The name of the DOM node as a String.
	*****************************************************************************************/   
	public String getNodeName() { return(((Node)userObject).getNodeName()); }

	/*****************************************************************************************
	* Method used to return the value of the DOM node in the userObject.
	* @return The value of the DOM node as a String
	*****************************************************************************************/   
	public String getNodeValue() {
		return (decodeString(((Node)userObject).getNodeValue()));
	}

	/*****************************************************************************************
	* Method used to set the value of the userObject(DOM Node).  This method also messages
	* the data model for which this node is a part of so that any viewers(DIDTree) that are 
	* using the data model can update themselves accordingly.
	* @param newValue A String representing the new value of the DOM Node(userObject).
	*****************************************************************************************/   
	public void setNodeValue(String newValue) {
		if (!(newValue.equals(((Node)userObject).getNodeValue()))) {
			((Node)userObject).setNodeValue(newValue);
			myModel.nodeChanged(this);
		}
	}

	/*****************************************************************************************
	* Method used to return the type of Node the userOject represents.  The integer return is 
	* specific to the DOM, i.e. ELEMENT, ATTRIBUTE, etc.  Note: This method is provided for 
	* completeness and convience.
	* @return An integer representing the type of DOM Node the userObject belongs to.  For 
	* example: ELEMENT, ATTRIBUTE, etc.
	*****************************************************************************************/   
	public int getNodeType() { return (((Node)userObject).getNodeType()); }

	/*****************************************************************************************
	* Method used to determine the type of data the userObject contains as is pertains to the
	* application.  This is accomplished by evaluating the name of the userObject and 
	* assigning a value (A constant defined in the attributes i.e. PAGE) to the dataType
	* attribute.
	*****************************************************************************************/   
	private void setDataType() {
		Node tmpNode = (Node)userObject;
		String tmpString = tmpNode.getNodeName();
		if (tmpString.equalsIgnoreCase(PAGE)) {
			dataType = PAGE;
		} else if (myModel.EntityType.isValid(tmpString)) {
			int index = myModel.EntityType.getIndex(tmpString);
			dataType = myModel.EntityType.getName(index);
		} else {
			dataType = OTHER;					
		}
	}

	/*****************************************************************************************
	* Method used to deterime the type of data this node userObject contains as is pertains to
	* this application.  The method returns the value stored in the dataType attribute.
	* @return A String representing the type of data this node's userObject contains.
	* @see #setDataType
	* @see #PAGE
	*****************************************************************************************/   
	public String getDataType() { return (dataType); }

	/*****************************************************************************************
	* Method used to return a Java String representation of this object.  In this 
	* implementation the representation of this object is derived from the contents of it's 
	* userObject.  This works well since the Java DefaultTreeCellRenderer uses toString()
	* method of the nodes in a JTree to construct a visual representation of the object.  Thus
	* this object is represented visually by the userObject's(DOM Node) name and value if it 
	* has one.
	* @return A String object representing this object.  In this implmentation that string 
	* consists of the userObject's name and the userObject's value (if it has one).
	*****************************************************************************************/   
	public String toString(){
		if (((Node)userObject).getNodeValue() != null) {
			return(((Node)userObject).getNodeName() + " - " + ((Node)userObject).getNodeValue());
		} else {
			return(((Node)userObject).getNodeName());
		}
	}

	/*****************************************************************************************
	* Method used to set a reference to ROI which represents this node.
	* @param newRoi New reference to an ROI.
	*****************************************************************************************/   
	public void setRoi(DIDRoiIFC newRoi) {
		myRoi = newRoi;
	}

	/*****************************************************************************************
	* Method used to return a reference to the ROI which is representing this Node in the 
	* ImageCanvas.
	* @return An implementation of the DIDRoiIFC which represents this Node.
	*****************************************************************************************/   
	public DIDRoiIFC getRoi() {
		return myRoi;
	}

	/*****************************************************************************************
	* Method used to return a reference to the TreeModel to which this Node belongs.
	* @return A DIDDefaultTreeModel object which refrences the TreeModel to which this Node 
	* belongs.
	*****************************************************************************************/   
	public DIDDefaultTreeModel getModel() {
		return myModel;
	}

	/*****************************************************************************************
	* Convenience method used to set a reference to the TreeModel to which this Node belongs.
	* @param newModel A DIDDefaultTreeModel object which refrences the TreeModel to which this
	* Node belongs.
	*****************************************************************************************/   
	public void setModel(DIDDefaultTreeModel newModel) {
		myModel = newModel;
	}
	
	/*****************************************************************************************
	* Method used to remove the userObject from the TreeModels DOM Tree.
	*****************************************************************************************/
	public void deleteDomNode() {
		Node nodeToDelete = (Node)userObject;
		Node parentNode = nodeToDelete.getParentNode();
		if (parentNode != null) {
			parentNode.removeChild(nodeToDelete);
			removeFromParent();
		} 
	}
	/** Method used to convert the unicode values into unicode string */
	public static String decodeString(String uniStr) {
		String newStr = new String();

		if (uniStr == null)  return null;

		for (int i=0; i < uniStr.length(); i++) {
			if(uniStr.substring(i).startsWith("\\u")) {
				String temp = uniStr.substring(i+2, i+6);
				newStr += (new Character((char)Integer.parseInt(temp, 16))).toString();
				i += 5;
			} else {
				newStr += uniStr.substring(i, i+1);
			}
		}

		return newStr;
	}
}
